from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from .common import _BulkAdExtensionBase
from .common import _BulkAdGroupAdExtensionAssociation
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

from bingads.v13.internal.extensions import *


_AppAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('AppAdExtension'))


class BulkAppAdExtension(_BulkAdExtensionBase):
    """ Represents a app ad extension.

    This class exposes the :attr:`app_ad_extension` property that can be read and written
    as fields of the App Ad Extension record in a bulk file.

    For more information, see App Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _AppAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'AppAdExtension'
            ))
        super(BulkAppAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )

    @property
    def app_ad_extension(self):
        """ The app ad extension.

        see App Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @app_ad_extension.setter
    def app_ad_extension(self, value):
        self._ad_extension = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.AppPlatform,
            field_to_csv=lambda c: c.app_ad_extension.AppPlatform,
            csv_to_field=lambda c, v: setattr(c.app_ad_extension, 'AppPlatform', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AppStoreId,
            field_to_csv=lambda c: c.app_ad_extension.AppStoreId,
            csv_to_field=lambda c, v: setattr(c.app_ad_extension, 'AppStoreId', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DestinationUrl,
            field_to_csv=lambda c: c.app_ad_extension.DestinationUrl,
            csv_to_field=lambda c, v: setattr(c.app_ad_extension, 'DestinationUrl', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: c.app_ad_extension.DisplayText,
            csv_to_field=lambda c, v: setattr(c.app_ad_extension, 'DisplayText', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.app_ad_extension.FinalUrlSuffix, c.app_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.app_ad_extension, 'FinalUrlSuffix', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.app_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('AppAdExtension')
        self.app_ad_extension.Type = 'AppAdExtension'
        super(BulkAppAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkAppAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.app_ad_extension, 'app_ad_extension')
        super(BulkAppAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkAppAdExtension._MAPPINGS)


class BulkAccountAppAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level app ad extension.

    This class exposes properties that can be read and written
    as fields of the Account App Ad Extension record in a bulk file.

    For more information, see Account App Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkCampaignAppAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level app ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign App Ad Extension record in a bulk file.

    For more information, see Campaign App Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkAdGroupAppAdExtension(_BulkAdGroupAdExtensionAssociation):
    """ Represents an ad group level App ad extension.

    This class exposes properties that can be read and written
    as fields of the Ad Group App Ad Extension record in a bulk file.

    For more information, see Ad Group App Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass



# coding: utf-8

# flake8: noqa
"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.reporting.account_performance_report_column import AccountPerformanceReportColumn
from openapi_client.models.reporting.account_performance_report_filter import AccountPerformanceReportFilter
from openapi_client.models.reporting.account_performance_report_request import AccountPerformanceReportRequest
from openapi_client.models.reporting.account_report_scope import AccountReportScope
from openapi_client.models.reporting.account_status_report_filter import AccountStatusReportFilter
from openapi_client.models.reporting.account_through_ad_group_report_scope import AccountThroughAdGroupReportScope
from openapi_client.models.reporting.account_through_asset_group_report_scope import AccountThroughAssetGroupReportScope
from openapi_client.models.reporting.account_through_campaign_report_scope import AccountThroughCampaignReportScope
from openapi_client.models.reporting.ad_api_error import AdApiError
from openapi_client.models.reporting.ad_api_fault_detail import AdApiFaultDetail
from openapi_client.models.reporting.ad_distribution_report_filter import AdDistributionReportFilter
from openapi_client.models.reporting.ad_dynamic_text_performance_report_column import AdDynamicTextPerformanceReportColumn
from openapi_client.models.reporting.ad_dynamic_text_performance_report_filter import AdDynamicTextPerformanceReportFilter
from openapi_client.models.reporting.ad_dynamic_text_performance_report_request import AdDynamicTextPerformanceReportRequest
from openapi_client.models.reporting.ad_extension_by_ad_report_column import AdExtensionByAdReportColumn
from openapi_client.models.reporting.ad_extension_by_ad_report_filter import AdExtensionByAdReportFilter
from openapi_client.models.reporting.ad_extension_by_ad_report_request import AdExtensionByAdReportRequest
from openapi_client.models.reporting.ad_extension_by_keyword_report_column import AdExtensionByKeywordReportColumn
from openapi_client.models.reporting.ad_extension_by_keyword_report_filter import AdExtensionByKeywordReportFilter
from openapi_client.models.reporting.ad_extension_by_keyword_report_request import AdExtensionByKeywordReportRequest
from openapi_client.models.reporting.ad_extension_detail_report_column import AdExtensionDetailReportColumn
from openapi_client.models.reporting.ad_extension_detail_report_filter import AdExtensionDetailReportFilter
from openapi_client.models.reporting.ad_extension_detail_report_request import AdExtensionDetailReportRequest
from openapi_client.models.reporting.ad_group_performance_report_column import AdGroupPerformanceReportColumn
from openapi_client.models.reporting.ad_group_performance_report_filter import AdGroupPerformanceReportFilter
from openapi_client.models.reporting.ad_group_performance_report_request import AdGroupPerformanceReportRequest
from openapi_client.models.reporting.ad_group_report_scope import AdGroupReportScope
from openapi_client.models.reporting.ad_group_status_report_filter import AdGroupStatusReportFilter
from openapi_client.models.reporting.ad_performance_report_column import AdPerformanceReportColumn
from openapi_client.models.reporting.ad_performance_report_filter import AdPerformanceReportFilter
from openapi_client.models.reporting.ad_performance_report_request import AdPerformanceReportRequest
from openapi_client.models.reporting.ad_status_report_filter import AdStatusReportFilter
from openapi_client.models.reporting.ad_type_report_filter import AdTypeReportFilter
from openapi_client.models.reporting.age_gender_audience_report_column import AgeGenderAudienceReportColumn
from openapi_client.models.reporting.age_gender_audience_report_filter import AgeGenderAudienceReportFilter
from openapi_client.models.reporting.age_gender_audience_report_request import AgeGenderAudienceReportRequest
from openapi_client.models.reporting.api_fault_detail import ApiFaultDetail
from openapi_client.models.reporting.application_fault import ApplicationFault
from openapi_client.models.reporting.apps_performance_report_column import AppsPerformanceReportColumn
from openapi_client.models.reporting.apps_performance_report_filter import AppsPerformanceReportFilter
from openapi_client.models.reporting.apps_performance_report_request import AppsPerformanceReportRequest
from openapi_client.models.reporting.asset_group_performance_report_column import AssetGroupPerformanceReportColumn
from openapi_client.models.reporting.asset_group_performance_report_filter import AssetGroupPerformanceReportFilter
from openapi_client.models.reporting.asset_group_performance_report_request import AssetGroupPerformanceReportRequest
from openapi_client.models.reporting.asset_group_report_scope import AssetGroupReportScope
from openapi_client.models.reporting.asset_group_status_report_filter import AssetGroupStatusReportFilter
from openapi_client.models.reporting.asset_performance_report_column import AssetPerformanceReportColumn
from openapi_client.models.reporting.asset_performance_report_request import AssetPerformanceReportRequest
from openapi_client.models.reporting.audience_performance_report_column import AudiencePerformanceReportColumn
from openapi_client.models.reporting.audience_performance_report_filter import AudiencePerformanceReportFilter
from openapi_client.models.reporting.audience_performance_report_request import AudiencePerformanceReportRequest
from openapi_client.models.reporting.batch_error import BatchError
from openapi_client.models.reporting.bid_match_type_report_filter import BidMatchTypeReportFilter
from openapi_client.models.reporting.bid_strategy_type_report_filter import BidStrategyTypeReportFilter
from openapi_client.models.reporting.budget_summary_report_column import BudgetSummaryReportColumn
from openapi_client.models.reporting.budget_summary_report_request import BudgetSummaryReportRequest
from openapi_client.models.reporting.call_detail_report_column import CallDetailReportColumn
from openapi_client.models.reporting.call_detail_report_filter import CallDetailReportFilter
from openapi_client.models.reporting.call_detail_report_request import CallDetailReportRequest
from openapi_client.models.reporting.campaign_performance_report_column import CampaignPerformanceReportColumn
from openapi_client.models.reporting.campaign_performance_report_filter import CampaignPerformanceReportFilter
from openapi_client.models.reporting.campaign_performance_report_request import CampaignPerformanceReportRequest
from openapi_client.models.reporting.campaign_report_scope import CampaignReportScope
from openapi_client.models.reporting.campaign_status_report_filter import CampaignStatusReportFilter
from openapi_client.models.reporting.campaign_type_report_filter import CampaignTypeReportFilter
from openapi_client.models.reporting.category_click_coverage_report_column import CategoryClickCoverageReportColumn
from openapi_client.models.reporting.category_click_coverage_report_filter import CategoryClickCoverageReportFilter
from openapi_client.models.reporting.category_click_coverage_report_request import CategoryClickCoverageReportRequest
from openapi_client.models.reporting.category_insights_report_column import CategoryInsightsReportColumn
from openapi_client.models.reporting.category_insights_report_filter import CategoryInsightsReportFilter
from openapi_client.models.reporting.category_insights_report_request import CategoryInsightsReportRequest
from openapi_client.models.reporting.change_entity_report_filter import ChangeEntityReportFilter
from openapi_client.models.reporting.change_type_report_filter import ChangeTypeReportFilter
from openapi_client.models.reporting.combination_performance_report_column import CombinationPerformanceReportColumn
from openapi_client.models.reporting.combination_performance_report_request import CombinationPerformanceReportRequest
from openapi_client.models.reporting.conversion_performance_report_column import ConversionPerformanceReportColumn
from openapi_client.models.reporting.conversion_performance_report_filter import ConversionPerformanceReportFilter
from openapi_client.models.reporting.conversion_performance_report_request import ConversionPerformanceReportRequest
from openapi_client.models.reporting.dsa_auto_target_performance_report_column import DSAAutoTargetPerformanceReportColumn
from openapi_client.models.reporting.dsa_auto_target_performance_report_filter import DSAAutoTargetPerformanceReportFilter
from openapi_client.models.reporting.dsa_auto_target_performance_report_request import DSAAutoTargetPerformanceReportRequest
from openapi_client.models.reporting.dsa_category_performance_report_column import DSACategoryPerformanceReportColumn
from openapi_client.models.reporting.dsa_category_performance_report_filter import DSACategoryPerformanceReportFilter
from openapi_client.models.reporting.dsa_category_performance_report_request import DSACategoryPerformanceReportRequest
from openapi_client.models.reporting.dsa_search_query_performance_report_column import DSASearchQueryPerformanceReportColumn
from openapi_client.models.reporting.dsa_search_query_performance_report_filter import DSASearchQueryPerformanceReportFilter
from openapi_client.models.reporting.dsa_search_query_performance_report_request import DSASearchQueryPerformanceReportRequest
from openapi_client.models.reporting.delivered_match_type_report_filter import DeliveredMatchTypeReportFilter
from openapi_client.models.reporting.destination_url_performance_report_column import DestinationUrlPerformanceReportColumn
from openapi_client.models.reporting.destination_url_performance_report_filter import DestinationUrlPerformanceReportFilter
from openapi_client.models.reporting.destination_url_performance_report_request import DestinationUrlPerformanceReportRequest
from openapi_client.models.reporting.device_os_report_filter import DeviceOSReportFilter
from openapi_client.models.reporting.device_type_report_filter import DeviceTypeReportFilter
from openapi_client.models.reporting.dynamic_ad_target_status_report_filter import DynamicAdTargetStatusReportFilter
from openapi_client.models.reporting.feed_item_performance_report_column import FeedItemPerformanceReportColumn
from openapi_client.models.reporting.feed_item_performance_report_request import FeedItemPerformanceReportRequest
from openapi_client.models.reporting.geographic_performance_report_column import GeographicPerformanceReportColumn
from openapi_client.models.reporting.geographic_performance_report_filter import GeographicPerformanceReportFilter
from openapi_client.models.reporting.geographic_performance_report_request import GeographicPerformanceReportRequest
from openapi_client.models.reporting.goals_and_funnels_report_column import GoalsAndFunnelsReportColumn
from openapi_client.models.reporting.goals_and_funnels_report_filter import GoalsAndFunnelsReportFilter
from openapi_client.models.reporting.goals_and_funnels_report_request import GoalsAndFunnelsReportRequest
from openapi_client.models.reporting.hotel_dimension_performance_report_column import HotelDimensionPerformanceReportColumn
from openapi_client.models.reporting.hotel_dimension_performance_report_filter import HotelDimensionPerformanceReportFilter
from openapi_client.models.reporting.hotel_dimension_performance_report_request import HotelDimensionPerformanceReportRequest
from openapi_client.models.reporting.hotel_group_performance_report_column import HotelGroupPerformanceReportColumn
from openapi_client.models.reporting.hotel_group_performance_report_filter import HotelGroupPerformanceReportFilter
from openapi_client.models.reporting.hotel_group_performance_report_request import HotelGroupPerformanceReportRequest
from openapi_client.models.reporting.keyword_performance_report_column import KeywordPerformanceReportColumn
from openapi_client.models.reporting.keyword_performance_report_filter import KeywordPerformanceReportFilter
from openapi_client.models.reporting.keyword_performance_report_request import KeywordPerformanceReportRequest
from openapi_client.models.reporting.keyword_performance_report_sort import KeywordPerformanceReportSort
from openapi_client.models.reporting.keyword_status_report_filter import KeywordStatusReportFilter
from openapi_client.models.reporting.language_report_filter import LanguageReportFilter
from openapi_client.models.reporting.model_date import ModelDate
from openapi_client.models.reporting.negative_keyword_conflict_report_column import NegativeKeywordConflictReportColumn
from openapi_client.models.reporting.negative_keyword_conflict_report_filter import NegativeKeywordConflictReportFilter
from openapi_client.models.reporting.negative_keyword_conflict_report_request import NegativeKeywordConflictReportRequest
from openapi_client.models.reporting.operation_error import OperationError
from openapi_client.models.reporting.poll_generate_report_request import PollGenerateReportRequest
from openapi_client.models.reporting.poll_generate_report_response import PollGenerateReportResponse
from openapi_client.models.reporting.product_dimension_performance_report_column import ProductDimensionPerformanceReportColumn
from openapi_client.models.reporting.product_dimension_performance_report_filter import ProductDimensionPerformanceReportFilter
from openapi_client.models.reporting.product_dimension_performance_report_request import ProductDimensionPerformanceReportRequest
from openapi_client.models.reporting.product_match_count_report_column import ProductMatchCountReportColumn
from openapi_client.models.reporting.product_match_count_report_request import ProductMatchCountReportRequest
from openapi_client.models.reporting.product_negative_keyword_conflict_report_column import ProductNegativeKeywordConflictReportColumn
from openapi_client.models.reporting.product_negative_keyword_conflict_report_filter import ProductNegativeKeywordConflictReportFilter
from openapi_client.models.reporting.product_negative_keyword_conflict_report_request import ProductNegativeKeywordConflictReportRequest
from openapi_client.models.reporting.product_partition_performance_report_column import ProductPartitionPerformanceReportColumn
from openapi_client.models.reporting.product_partition_performance_report_filter import ProductPartitionPerformanceReportFilter
from openapi_client.models.reporting.product_partition_performance_report_request import ProductPartitionPerformanceReportRequest
from openapi_client.models.reporting.product_partition_unit_performance_report_column import ProductPartitionUnitPerformanceReportColumn
from openapi_client.models.reporting.product_partition_unit_performance_report_filter import ProductPartitionUnitPerformanceReportFilter
from openapi_client.models.reporting.product_partition_unit_performance_report_request import ProductPartitionUnitPerformanceReportRequest
from openapi_client.models.reporting.product_search_query_performance_report_column import ProductSearchQueryPerformanceReportColumn
from openapi_client.models.reporting.product_search_query_performance_report_filter import ProductSearchQueryPerformanceReportFilter
from openapi_client.models.reporting.product_search_query_performance_report_request import ProductSearchQueryPerformanceReportRequest
from openapi_client.models.reporting.professional_demographics_audience_report_column import ProfessionalDemographicsAudienceReportColumn
from openapi_client.models.reporting.professional_demographics_audience_report_filter import ProfessionalDemographicsAudienceReportFilter
from openapi_client.models.reporting.professional_demographics_audience_report_request import ProfessionalDemographicsAudienceReportRequest
from openapi_client.models.reporting.publisher_usage_performance_report_column import PublisherUsagePerformanceReportColumn
from openapi_client.models.reporting.publisher_usage_performance_report_filter import PublisherUsagePerformanceReportFilter
from openapi_client.models.reporting.publisher_usage_performance_report_request import PublisherUsagePerformanceReportRequest
from openapi_client.models.reporting.report_aggregation import ReportAggregation
from openapi_client.models.reporting.report_format import ReportFormat
from openapi_client.models.reporting.report_request import ReportRequest
from openapi_client.models.reporting.report_request_status import ReportRequestStatus
from openapi_client.models.reporting.report_request_status_type import ReportRequestStatusType
from openapi_client.models.reporting.report_time import ReportTime
from openapi_client.models.reporting.report_time_period import ReportTimePeriod
from openapi_client.models.reporting.report_time_zone import ReportTimeZone
from openapi_client.models.reporting.search_campaign_change_history_report_column import SearchCampaignChangeHistoryReportColumn
from openapi_client.models.reporting.search_campaign_change_history_report_filter import SearchCampaignChangeHistoryReportFilter
from openapi_client.models.reporting.search_campaign_change_history_report_request import SearchCampaignChangeHistoryReportRequest
from openapi_client.models.reporting.search_insight_performance_report_column import SearchInsightPerformanceReportColumn
from openapi_client.models.reporting.search_insight_performance_report_filter import SearchInsightPerformanceReportFilter
from openapi_client.models.reporting.search_insight_performance_report_request import SearchInsightPerformanceReportRequest
from openapi_client.models.reporting.search_query_performance_report_column import SearchQueryPerformanceReportColumn
from openapi_client.models.reporting.search_query_performance_report_filter import SearchQueryPerformanceReportFilter
from openapi_client.models.reporting.search_query_performance_report_request import SearchQueryPerformanceReportRequest
from openapi_client.models.reporting.share_of_voice_report_column import ShareOfVoiceReportColumn
from openapi_client.models.reporting.share_of_voice_report_filter import ShareOfVoiceReportFilter
from openapi_client.models.reporting.share_of_voice_report_request import ShareOfVoiceReportRequest
from openapi_client.models.reporting.sort_order import SortOrder
from openapi_client.models.reporting.submit_generate_report_request import SubmitGenerateReportRequest
from openapi_client.models.reporting.submit_generate_report_response import SubmitGenerateReportResponse
from openapi_client.models.reporting.travel_query_insight_report_column import TravelQueryInsightReportColumn
from openapi_client.models.reporting.travel_query_insight_report_request import TravelQueryInsightReportRequest
from openapi_client.models.reporting.user_location_performance_report_column import UserLocationPerformanceReportColumn
from openapi_client.models.reporting.user_location_performance_report_filter import UserLocationPerformanceReportFilter
from openapi_client.models.reporting.user_location_performance_report_request import UserLocationPerformanceReportRequest



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from .bulk_audience import BulkAudience

class BulkCustomAudience(BulkAudience):
    """ Represents a Custom Audience that can be read or written in a bulk file.

    This class exposes the :attr:`custom_audience` property that can be read and written as fields of the
    Custom Audience record in a bulk file.

    For more information, see Custom Audience at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 custom_audience=None,
                 status=None):
        super(BulkCustomAudience, self).__init__(audience = custom_audience, status = status)

    @property
    def custom_audience(self):
        """ Defines a Custom Audience """

        return self._audience

    @custom_audience.setter
    def custom_audience(self, custom_audience):
        self._audience = custom_audience

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.custom_audience, 'custom_audience')
        super(BulkCustomAudience, self).process_mappings_to_row_values(row_values, exclude_readonly)

    def process_mappings_from_row_values(self, row_values):
        self.custom_audience = _CAMPAIGN_OBJECT_FACTORY_V13.create('CustomAudience')
        super(BulkCustomAudience, self).process_mappings_from_row_values(row_values)




from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

from. import BulkAdCustomizerAttributeEntityBase


class BulkCampaignAdCustomizerAttribute(BulkAdCustomizerAttributeEntityBase):
    """ Represents a CampaignAdCustomizerAttribute.
    
    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, id = None, name=None, parent_id=None, attribute_value=None, editorial_status = None):
        super(BulkCampaignAdCustomizerAttribute, self).__init__(id, name, parent_id, attribute_value, editorial_status)


    @property
    def campaign_id(self):
        """ the campaign id of bulk record
        Corresponds to the 'ParentId' field in the bulk file.

        :rtype: str
        """
        return self._parent_id

    @campaign_id.setter
    def campaign_id(self, value):
        self._parent_id = value        




__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .reporting_download_parameters import *
from .exceptions import *
from .reporting_operation import *
from .reporting_operation_status import *
from .reporting_service_manager import *
from .report_contract import *
from .report_file_reader import *


from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.bulk.entities.target_criterions.bulk_campaign_biddable_criterion import BulkCampaignBiddableCriterion
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from abc import ABCMeta, abstractmethod

class BulkCampaignProfileCriterion(BulkCampaignBiddableCriterion):
    """ The base class for campaign level profile criterion that can be read or written in a bulk file.

    This class exposes the :attr:`biddable_campaign_criterion` property that can be read and written in a bulk file.

    For more information, see Bulk File Schema at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 biddable_campaign_criterion=None,
                 campaign_name=None, ):
        super(BulkCampaignProfileCriterion, self).__init__(biddable_campaign_criterion, campaign_name)

    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.Profile,
            field_to_csv=lambda c: c.profile_name,
            csv_to_field=lambda c, v: setattr(c, 'profile_name', v)
        ),
        _SimpleBulkMapping(
            _StringTable.ProfileId,
            field_to_csv=lambda c: bulk_str(c.biddable_campaign_criterion.Criterion.ProfileId),
            csv_to_field=lambda c, v: setattr(c.biddable_campaign_criterion.Criterion, 'ProfileId', int(v) if v else None)
        ),
    ]
    
    def create_criterion(self):
        self._biddable_campaign_criterion.Criterion = _CAMPAIGN_OBJECT_FACTORY_V13.create('ProfileCriterion')
        self._biddable_campaign_criterion.Criterion.ProfileType = self.profile_type()
        self._biddable_campaign_criterion.Criterion.Type = 'ProfileCriterion'

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(BulkCampaignProfileCriterion, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkCampaignProfileCriterion._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        super(BulkCampaignProfileCriterion, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkCampaignProfileCriterion._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkCampaignProfileCriterion, self).read_additional_data(stream_reader)

    @abstractmethod
    def profile_type(self):
        pass

class BulkCampaignCompanyNameCriterion(BulkCampaignProfileCriterion):
    """ Represents an Campaign CompanyName Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`biddable_campaign_criterion` property that can be read and written as fields of the
    Campaign CompanyName Criterion record in a bulk file.

    For more information, see Campaign CompanyName Criterion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 biddable_campaign_criterion=None,
                 campaign_name=None):
        super(BulkCampaignCompanyNameCriterion, self).__init__()

        self._biddable_campaign_criterion = biddable_campaign_criterion
        self._campaign_name = campaign_name

    def profile_type(self):
        return 'CompanyName'

class BulkCampaignIndustryCriterion(BulkCampaignProfileCriterion):
    """ Represents an Campaign Industry Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`biddable_campaign_criterion` property that can be read and written as fields of the
    Campaign Industry Criterion record in a bulk file.

    For more information, see Campaign Industry Criterion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 biddable_campaign_criterion=None,
                 campaign_name=None):
        super(BulkCampaignIndustryCriterion, self).__init__()

        self._biddable_campaign_criterion = biddable_campaign_criterion
        self._campaign_name = campaign_name

    def profile_type(self):
        return 'Industry'

class BulkCampaignJobFunctionCriterion(BulkCampaignProfileCriterion):
    """ Represents an Campaign JobFunction Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`biddable_campaign_criterion` property that can be read and written as fields of the
    Campaign JobFunction Criterion record in a bulk file.

    For more information, see Campaign JobFunction Criterion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 biddable_campaign_criterion=None,
                 campaign_name=None):
        super(BulkCampaignJobFunctionCriterion, self).__init__()

        self._biddable_campaign_criterion = biddable_campaign_criterion
        self._campaign_name = campaign_name

    def profile_type(self):
        return 'JobFunction'


# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class CurrencyCode(str, Enum):
    AED = 'AED'

    ALL = 'ALL'

    AMD = 'AMD'

    ARS = 'ARS'

    AUD = 'AUD'

    AZM = 'AZM'

    BGL = 'BGL'

    BHD = 'BHD'

    BND = 'BND'

    BOB = 'BOB'

    BRL = 'BRL'

    BYB = 'BYB'

    BZD = 'BZD'

    CAD = 'CAD'

    CHF = 'CHF'

    CLP = 'CLP'

    CNY = 'CNY'

    COP = 'COP'

    CRC = 'CRC'

    CZK = 'CZK'

    DOP = 'DOP'

    DKK = 'DKK'

    DZD = 'DZD'

    EEK = 'EEK'

    EGP = 'EGP'

    EUR = 'EUR'

    GBP = 'GBP'

    GEL = 'GEL'

    GTQ = 'GTQ'

    HKD = 'HKD'

    HNL = 'HNL'

    HRK = 'HRK'

    HUF = 'HUF'

    IDR = 'IDR'

    ILS = 'ILS'

    INR = 'INR'

    IQD = 'IQD'

    IRR = 'IRR'

    ISK = 'ISK'

    JMD = 'JMD'

    JOD = 'JOD'

    JPY = 'JPY'

    KES = 'KES'

    KGS = 'KGS'

    KRW = 'KRW'

    KWD = 'KWD'

    KZT = 'KZT'

    LBP = 'LBP'

    LTL = 'LTL'

    LVL = 'LVL'

    LYD = 'LYD'

    MAD = 'MAD'

    MKD = 'MKD'

    MOP = 'MOP'

    MNT = 'MNT'

    MVR = 'MVR'

    MXN = 'MXN'

    MYR = 'MYR'

    NIO = 'NIO'

    NOK = 'NOK'

    NZD = 'NZD'

    OMR = 'OMR'

    PAB = 'PAB'

    PEN = 'PEN'

    PHP = 'PHP'

    PKR = 'PKR'

    PLN = 'PLN'

    PYG = 'PYG'

    QAR = 'QAR'

    ROL = 'ROL'

    RUR = 'RUR'

    SAR = 'SAR'

    SEK = 'SEK'

    SGD = 'SGD'

    SIT = 'SIT'

    SKK = 'SKK'

    SYP = 'SYP'

    THB = 'THB'

    TND = 'TND'

    TRY = 'TRY'

    TTD = 'TTD'

    TWD = 'TWD'

    UAH = 'UAH'

    USD = 'USD'

    UYU = 'UYU'

    UZS = 'UZS'

    VEF = 'VEF'

    YER = 'YER'

    VND = 'VND'

    YUN = 'YUN'

    ZAR = 'ZAR'

    ZWD = 'ZWD'

    NGN = 'NGN'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CurrencyCode from a JSON string"""
        return cls(json.loads(json_str))





import re
import time
import contextlib
import ssl
import requests
import zipfile
import os
import sys
import shutil

from openapi_client.models.reporting import *
from .reporting_operation_status import *
from .exceptions import *
from bingads.util import _PollingBlocker, errorcode_of_exception, ratelimit_retry_duration
from bingads.exceptions import *

from ...service_client import ServiceClient
from ...manifest import *

from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager


class TlsHttpAdapter(HTTPAdapter):
    """" Transport adapter that chooses the TLS protocols based on python versions. """

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = PoolManager(
            num_pools=connections,
            maxsize=maxsize,
            block=block,
            ssl_version=ssl.PROTOCOL_SSLv23
        )


class ReportingDownloadOperation(object):
    """ Represents a reporting download operation requested by a user.

    You can use this class to poll for the download status, and then download the file when available.

    *Example:*

    The :meth:`.ReportingServiceManager.submit_download` method returns an instance of this class.
    If for any reason you do not want to wait for the file to be prepared for download,
    for example if your application quits unexpectedly or you have other tasks to process, you can
    use an instance of :class:`.ReportingDownloadOperation` to download the file when it is available.
    """

    def __init__(self,
                 request_id,
                 authorization_data,
                 poll_interval_in_milliseconds=5000,
                 environment='production',
                 tracking_id=None,
                 **suds_options):
        self._request_id = request_id
        self._service_client = ServiceClient('Reporting', 13, authorization_data, environment, **suds_options)
        self._authorization_data = authorization_data
        self._poll_interval_in_milliseconds = poll_interval_in_milliseconds
        self._final_status = None
        self.tracking_id=tracking_id

    def download_result_file(self, result_file_directory, result_file_name, decompress, overwrite, timeout_in_milliseconds=None):
        """ Download file with specified URL and download parameters.

        :param result_file_directory: The download result local directory name.
        :type result_file_directory: str
        :param result_file_name: The download result local file name.
        :type result_file_name: str | None
        :param decompress: Determines whether to decompress the ZIP file.
                            If set to true, the file will be decompressed after download.
                            The default value is false, in which case the downloaded file is not decompressed.
        :type decompress: bool
        :param overwrite: Indicates whether the result file should overwrite the existing file if any.
        :type overwrite: bool
        :return: The download file path.
        :rtype: str
        :param timeout_in_milliseconds: (optional) timeout for download result file in milliseconds
        :type timeout_in_milliseconds: int
        """

        if result_file_directory is None:
            raise ValueError('result_file_directory cannot be None.')

        url = self.final_status.report_download_url

        if url is None or url == '':
            return None

        if result_file_name is None:
            result_file_name = self.request_id

        if decompress:
            name, ext = os.path.splitext(result_file_name)
            if ext == '.zip':
                raise ValueError("Result file can't be decompressed into a file with extension 'zip'."
                                 " Please change the extension of the result_file_name or pass decompress_result_file = false")
            zip_file_path = os.path.join(result_file_directory, name + '.zip')
            result_file_path = os.path.join(result_file_directory, result_file_name)
        else:
            result_file_path = os.path.join(result_file_directory, result_file_name)
            zip_file_path = result_file_path

        if os.path.exists(result_file_path) and overwrite is False:
            raise FileExistsError('Result file: {0} exists'.format(result_file_path))
        headers = {
            'User-Agent': USER_AGENT,
        }

        with requests.Session() as s:
            s.mount('https://', TlsHttpAdapter())
            timeout_seconds = None if timeout_in_milliseconds is None else timeout_in_milliseconds / 1000.0
            try:
                r = s.get(url, headers=headers, stream=True, verify=True, timeout=timeout_seconds)
            except requests.Timeout as ex:
                raise FileDownloadException(ex)
            r.raise_for_status()
            try:
                with open(zip_file_path, 'wb') as f:
                    for chunk in r.iter_content(chunk_size=4096):
                        if chunk:
                            f.write(chunk)
                            f.flush()
                if decompress:
                    with contextlib.closing(zipfile.ZipFile(zip_file_path)) as compressed:
                        first = compressed.namelist()[0]
                        with open(result_file_path, 'wb') as f, compressed.open(first, 'r') as cc:
                            shutil.copyfileobj(cc, f)
            except Exception as ex:
                raise ex
            finally:
                if decompress and os.path.exists(zip_file_path):
                    os.remove(zip_file_path)
        return result_file_path

    def track(self, timeout_in_milliseconds=None):
        """ Runs until the reporting service has finished processing the download or upload request.

        :param timeout_in_milliseconds: (optional) timeout for tracking reporting download operation
        :type timeout_in_milliseconds: int
        :return: The final ReportingOperationStatus.
        :rtype: ReportingOperationStatus
        """

        if self.final_status is not None:
            return self.final_status
        blocker = _PollingBlocker(self.poll_interval_in_milliseconds, timeout_in_milliseconds)
        blocker.wait()
        while True:
            status = self.get_status()
            if status.status == 'Pending':
                blocker.wait()
                continue
            if status.status != 'Success':
                raise ReportingException('Exceptions while reporting download.', status.status)
            self._final_status = status
            return self._final_status

    def get_status(self):
        """ Track the detailed download status.

        :return: The status of reporting download operation.
        :rtype: ReportingOperationStatus
        """
        if self.final_status is not None:
            return self.final_status
        response = self._get_status_with_retry(4)
        headers = self.service_client.get_response_header()
        self.tracking_id = headers['TrackingId'] if 'TrackingId' in headers else None
        status = ReportingOperationStatus(
            status=response.ReportRequestStatus.Status,
            report_download_url=response.ReportRequestStatus.ReportDownloadUrl
            )
        if status.status == 'Success' or \
                status.status == 'Error':
            self._final_status = status
        return status

    def _get_status_with_retry(self, retry_times):
        while retry_times > 1:
            try:
                match = re.search(r"report_request_id='(.*?)'", str(self.request_id))
                request_id = match.group(1) if match else self.request_id
                poll_generate_report_request = PollGenerateReportRequest(
                    report_request_id=str(request_id)
                )
                return self.service_client.PollGenerateReport(poll_generate_report_request)
            except Exception as ex:
                retry_times -= 1
                if '117' == errorcode_of_exception(ex):
                    time.sleep(ratelimit_retry_duration[3 - retry_times])
                else:
                    time.sleep(1)
        match = re.search(r"report_request_id='(.*?)'", str(self.request_id))
        request_id = match.group(1) if match else self.request_id
        poll_generate_report_request = PollGenerateReportRequest(
            report_request_id=str(request_id)
        )
        return self.service_client.PollGenerateReport(poll_generate_report_request)

    @property
    def request_id(self):
        """ The request identifier corresponding to the reporting download, depending on the derived type.

        :rtype: str
        """

        return self._request_id

    @property
    def final_status(self):
        """ Gets the final status of the reporting operation or null if the operation is still running.

        :rtype: ReportingOperationStatus
        """

        return self._final_status

    @property
    def service_client(self):
        """ The internal reporting service client.

        :rtype: ServiceClient
        """

        return self._service_client

    @property
    def poll_interval_in_milliseconds(self):
        """ The time interval in milliseconds between two status polling attempts.

        :rtype: int
        """

        return self._poll_interval_in_milliseconds

    @poll_interval_in_milliseconds.setter
    def poll_interval_in_milliseconds(self, poll_interval):
        self._poll_interval_in_milliseconds = poll_interval



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.audience_condition import AudienceCondition
from openapi_client.models.campaign.conversion_value_rule_operator import ConversionValueRuleOperator
from openapi_client.models.campaign.conversion_value_rule_status import ConversionValueRuleStatus
from openapi_client.models.campaign.device_condition import DeviceCondition
from openapi_client.models.campaign.location_condition import LocationCondition
from typing_extensions import Self

class ConversionValueRule(BaseModel):
    """
    ConversionValueRule
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    location_condition: Optional[LocationCondition] = Field(default=None, alias="LocationCondition")
    audience_condition: Optional[AudienceCondition] = Field(default=None, alias="AudienceCondition")
    device_condition: Optional[DeviceCondition] = Field(default=None, alias="DeviceCondition")
    value: Optional[StrictFloat] = Field(default=None, alias="Value")
    operation: Optional[ConversionValueRuleOperator] = Field(default=None, alias="Operation")
    currency_code: Optional[StrictStr] = Field(default=None, alias="CurrencyCode")
    status: Optional[ConversionValueRuleStatus] = Field(default=None, alias="Status")
    __properties: ClassVar[List[str]] = ["Id", "Name", "LocationCondition", "AudienceCondition", "DeviceCondition", "Value", "Operation", "CurrencyCode", "Status"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of location_condition
        if self.location_condition:
            _dict['LocationCondition'] = self.location_condition.to_dict()
        # override the default output from pydantic by calling `to_dict()` of audience_condition
        if self.audience_condition:
            _dict['AudienceCondition'] = self.audience_condition.to_dict()
        # override the default output from pydantic by calling `to_dict()` of device_condition
        if self.device_condition:
            _dict['DeviceCondition'] = self.device_condition.to_dict()
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if location_condition (nullable) is None
        # and model_fields_set contains the field
        if self.location_condition is None and "location_condition" in self.model_fields_set:
            _dict['LocationCondition'] = None

        # set to None if audience_condition (nullable) is None
        # and model_fields_set contains the field
        if self.audience_condition is None and "audience_condition" in self.model_fields_set:
            _dict['AudienceCondition'] = None

        # set to None if device_condition (nullable) is None
        # and model_fields_set contains the field
        if self.device_condition is None and "device_condition" in self.model_fields_set:
            _dict['DeviceCondition'] = None

        # set to None if value (nullable) is None
        # and model_fields_set contains the field
        if self.value is None and "value" in self.model_fields_set:
            _dict['Value'] = None

        # set to None if operation (nullable) is None
        # and model_fields_set contains the field
        if self.operation is None and "operation" in self.model_fields_set:
            _dict['Operation'] = None

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['CurrencyCode'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ConversionValueRule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "LocationCondition": LocationCondition.from_dict(obj["LocationCondition"]) if obj.get("LocationCondition") is not None else None,
                        "AudienceCondition": AudienceCondition.from_dict(obj["AudienceCondition"]) if obj.get("AudienceCondition") is not None else None,
                        "DeviceCondition": DeviceCondition.from_dict(obj["DeviceCondition"]) if obj.get("DeviceCondition") is not None else None,
                        "Value": obj.get("Value") if obj.get("Value") is not None else None,
                        "Operation": obj.get("Operation") if obj.get("Operation") is not None else None,
                        "CurrencyCode": obj.get("CurrencyCode") if obj.get("CurrencyCode") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None
        })
        return _obj



import tempfile
import uuid
import codecs
import csv
import io

from openapi_client import DataScope
from .bulk_operation import *
from .upload_parameters import *
from .file_reader import *
from .file_writer import *
from bingads.manifest import *
from bingads.service_client import ServiceClient
from bingads.authorization import *
from bingads.util import _TimeHelper
from bingads.exceptions import TimeoutException

class BulkServiceManager:
    SYNC_THRESHOLD = 1000
    BOMLEN = len(codecs.BOM_UTF8)
    """ Provides high level methods for uploading and downloading entities using the Bulk API functionality.

    Also provides methods for submitting upload or download operations.

    *Example:*

    :func:`download_file` will submit the download request to the bulk service,
    poll until the status is completed (or returns an error), and downloads the file locally.
    If instead you want to manage the low level details you would first call :func:`submit_download`,
    wait for the results file to be prepared using either :meth:`.BulkDownloadOperation.get_status`
    or :meth:`.BulkDownloadOperation.track`, and then download the file with the
    :meth:`.BulkOperation.download_result_file` method.
    """

    def __init__(self, authorization_data, poll_interval_in_milliseconds=5000, environment='production', working_directory=None, location=None, **suds_options):
        """ Initialize a new instance of this class.

        :param authorization_data: Represents a user who intends to access the corresponding customer and account.
        :type authorization_data: AuthorizationData
        :param environment: (optional) Represents which API environment to use, default is `production`, you can also pass `sandbox` in
        :type environment: str
        :param poll_interval_in_milliseconds: (optional) The time interval in milliseconds between two status polling attempts.
                                                         The default value is 15000 milliseconds.
        :type poll_interval_in_milliseconds: int
        :param working_directory: (optional)  Directory for storing temporary files needed for some operations
                                    (for example :func:`upload_entities` creates a temporary upload file).
        :param suds_options: The suds options need to pass to suds client
        """

        self._environment = environment
        self._service_client = ServiceClient('Bulk', 13, authorization_data, environment, location, **suds_options)
        self._location = location
        self._authorization_data = authorization_data
        self._poll_interval_in_milliseconds = poll_interval_in_milliseconds
        self._working_directory = os.path.join(tempfile.gettempdir(), WORKING_NAME)
        if working_directory is not None:
            self._working_directory = working_directory
        # make sure the working directory exists or create it.
        if not os.path.exists(self._working_directory):
            os.makedirs(self._working_directory)
        self._suds_options = suds_options

    def download_file(self, download_parameters, progress=None):
        """ Downloads the specified Bulk entities to a local file.

        :param download_parameters: Determines various download parameters, for example where the file should be downloaded.
        :type download_parameters: DownloadParameters
        :param progress: (optional) Tracking the percent complete progress information for the bulk operation.
        :type progress: BulkOperationProgressInfo -> None
        :return: The downloaded local bulk file path.
        :rtype: str
        """

        start_timestamp = _TimeHelper.get_current_time_milliseconds()
        operation = self.submit_download(download_parameters._submit_download_parameter)
        try:
            operation.track(progress, download_parameters.timeout_in_milliseconds)
        except TimeoutException:
            raise BulkDownloadException("Bulk file download tracking status timeout.")
        result_file_directory = self.working_directory
        if download_parameters.result_file_directory is not None:
            result_file_directory = download_parameters.result_file_directory
        download_result_file_timeout = _TimeHelper.get_remaining_time_milliseconds_with_min_value(start_timestamp, download_parameters.timeout_in_milliseconds)
        result_file_path = operation.download_result_file(
            result_file_directory=result_file_directory,
            result_file_name=download_parameters.result_file_name,
            decompress=download_parameters.decompress_result_file,
            overwrite=download_parameters.overwrite_result_file,
            timeout_in_milliseconds=download_result_file_timeout,
        )
        return result_file_path

    def download_entities(self, download_parameters, progress=None):
        """ Downloads the specified Bulk entities.

        :param download_parameters: Determines various download parameters, for example where the file should be downloaded.
        :type download_parameters: DownloadParameters
        :param progress: (optional) Tracking the percent complete progress information for the bulk operation.
        :type progress: BulkOperationProgressInfo -> None
        :return: Bulk entity generator.
        :rtype: generator[BulkEntity]
        """

        result_file_path = self.download_file(download_parameters, progress)
        result_file_type = ResultFileType.full_download \
            if download_parameters.last_sync_time_in_utc is None \
            else ResultFileType.partial_download
        with BulkFileReader(
            file_path=result_file_path,
            result_file_type=result_file_type,
            file_type=download_parameters.file_type,
        ) as reader:
            for entity in reader:
                yield entity

    def upload_file(self, file_upload_parameters, progress=None):
        """ Uploads the specified Bulk file.

        :param file_upload_parameters: Determines various upload parameters.
        :type file_upload_parameters: FileUploadParameters
        :param progress: (optional) Tracking the percent complete progress information for the bulk operation.
        :type progress: BulkOperationProgressInfo -> None
        :return: The download local bulk file path.
        :rtype: str
        """

        file_upload_parameters._submit_upload_parameters.timeout_in_milliseconds = file_upload_parameters.timeout_in_milliseconds
        operation = self.submit_upload(file_upload_parameters._submit_upload_parameters)
        return self.download_upload_result(operation, file_upload_parameters, progress)
    
    def download_upload_result(self, operation, file_upload_parameters, progress=None):
        start_timestamp = _TimeHelper.get_current_time_milliseconds()
        upload_operation_timeout = _TimeHelper.get_remaining_time_milliseconds_with_min_value(start_timestamp, file_upload_parameters.timeout_in_milliseconds)
        try:
            operation.track(progress, upload_operation_timeout)
        except TimeoutException:
            raise BulkUploadException("Bulk file upload tracking status timeout.")
        result_file_directory = self.working_directory
        if file_upload_parameters.result_file_directory is not None:
            result_file_directory = file_upload_parameters.result_file_directory
        download_result_file_timeout = _TimeHelper.get_remaining_time_milliseconds_with_min_value(start_timestamp, file_upload_parameters.timeout_in_milliseconds)
        result_file_path = operation.download_result_file(
            result_file_directory=result_file_directory,
            result_file_name=file_upload_parameters.result_file_name,
            decompress=file_upload_parameters.decompress_result_file,
            overwrite=file_upload_parameters.overwrite_result_file,
            timeout_in_milliseconds=download_result_file_timeout,
        )
        return result_file_path
    
    def need_to_try_upload_entity_records_sync_first(self, entity_upload_parameters):
        return len(entity_upload_parameters.entities) <= BulkServiceManager.SYNC_THRESHOLD

    def bulkupload_entities(self, entity_upload_parameters, tmp_file, progress=None):
        """ Uploads the specified Bulk entities in async way.

        :param entity_upload_parameters: Determines various upload parameters, for example what entities to upload.
        :type entity_upload_parameters: EntityUploadParameters
        :param tmp_file: The temp file path that contains the content to upload
        :type tmp_file: string 
        :param progress: (optional) Tracking the percent complete progress information for the bulk operation.
        :type progress: BulkOperationProgressInfo -> None
        :return: Bulk entity generator.
        :rtype: generator[BulkEntity]
        """

        file_upload_parameters = FileUploadParameters(
            upload_file_path=tmp_file,
            result_file_directory=entity_upload_parameters.result_file_directory,
            result_file_name=entity_upload_parameters.result_file_name,
            overwrite_result_file=entity_upload_parameters.overwrite_result_file,
            response_mode=entity_upload_parameters.response_mode,
            compress_upload_file=True,
        )
        result_file_path = self.upload_file(
            file_upload_parameters=file_upload_parameters,
            progress=progress,
        )
        with BulkFileReader(result_file_path, result_file_type=ResultFileType.upload) as reader:
            for entity in reader:
                yield entity
    
    def bulkupload_entitie_records(self, entity_upload_parameters, tmp_file, progress=None):
        """ Uploads the specified Bulk entities in sync way by UploadEntityRecords.
        """
        from openapi_client.models.bulk import UploadEntityRecordsRequest
        
        tmp_csv_file = io.open(tmp_file, encoding='utf-8-sig')
        entity_records = [x.strip() for x in tmp_csv_file.readlines()]
        
        try:
            request = UploadEntityRecordsRequest(
                account_id=self._authorization_data.account_id,
                entity_records=entity_records,
                response_mode=entity_upload_parameters.response_mode
            )
            
            response = self.service_client.upload_entity_records(
                upload_entity_records_request=request
            )
            if self.need_to_fall_back_to_async(response):
                headers = self.service_client.get_response_header()
                operation = BulkUploadOperation(
                    request_id=response.RequestId,
                    authorization_data=self._authorization_data,
                    poll_interval_in_milliseconds=self._poll_interval_in_milliseconds,
                    environment=self._environment,
                    tracking_id=headers['TrackingId'] if 'TrackingId' in headers else None,
                    location=self._location,
                    **self.suds_options
                    )
                file_path = self.download_upload_result(operation, entity_upload_parameters, progress)
                return self.read_result_from_bulk_file(file_path)
            else:
                return self.read_bulkupsert_response(response) 
        except Exception as ex:
            if 'OperationNotSupported' == operation_errorcode_of_exception(ex):
                return self.bulkupload_entities(entity_upload_parameters, tmp_file, progress)
            else:
                raise ex
            
    def need_to_fall_back_to_async(self, response):
        return response.RequestId is not None and \
            len(response.RequestId) > 0 and \
            response.RequestStatus == 'InProgress'
            
    def read_result_from_bulk_file(self, result_bulk_file):
        with BulkFileReader(result_bulk_file, result_file_type=ResultFileType.upload) as reader:
            for entity in reader:
                yield entity
            
    def read_bulkupsert_response(self, response):        
        with BulkRowsReader(response.EntityRecords) as reader:
            for entity in reader:
                yield entity
    
    def retry_with_BulkUpload(self, bulkupsert_response):
        if bulkupsert_response.Errors is not None:
            error_codes = [e.ErrorCode for e in bulkupsert_response.Errors]
            return FailedBulkUpsertRetryBulkUploadInstead in error_codes
        return False

    def upload_entities(self, entity_upload_parameters, progress=None):
        """ Uploads the specified Bulk entities.

        :param entity_upload_parameters: Determines various upload parameters, for example what entities to upload.
        :type entity_upload_parameters: EntityUploadParameters
        :param progress: (optional) Tracking the percent complete progress information for the bulk operation.
        :type progress: BulkOperationProgressInfo -> None
        :return: Bulk entity generator.
        :rtype: generator[BulkEntity]
        """
        
        tmp_file = path.join(self.working_directory, '{0}.csv'.format(uuid.uuid1()))
        with BulkFileWriter(tmp_file) as writer:
            for entity in entity_upload_parameters.entities:
                writer.write_entity(entity)

        if (self.need_to_try_upload_entity_records_sync_first(entity_upload_parameters)):
            return self.bulkupload_entitie_records(entity_upload_parameters, tmp_file, progress)
        else:
            return self.bulkupload_entities(entity_upload_parameters, tmp_file, progress)

    def submit_download(self, submit_download_parameters):
        """ Submits a download request to the Bing Ads bulk service with the specified parameters.

        :param submit_download_parameters: Determines various download parameters, for example what entities to download.
        :type submit_download_parameters: SubmitDownloadParameters
        :return: The submitted download operation
        :rtype: BulkDownloadOperation
        """

        data_scope = None if submit_download_parameters.data_scope is None else ' '.join(
            submit_download_parameters.data_scope)
        download_file_type = submit_download_parameters.file_type
        if submit_download_parameters.campaign_ids is None:
            from openapi_client.models.bulk import (
                DownloadCampaignsByAccountIdsRequest,
                DownloadEntity
            )
            
            request = DownloadCampaignsByAccountIdsRequest(
                account_ids=[self._authorization_data.account_id],
                data_scope=DataScope.ENTITYDATA,
                download_file_type=download_file_type,
                download_entities=submit_download_parameters.download_entities,
                format_version=BULK_FORMAT_VERSION_6,
                last_sync_time_in_utc=submit_download_parameters.last_sync_time_in_utc
            )
            
            response = self.service_client.download_campaigns_by_account_ids(
                download_campaigns_by_account_ids_request=request
            )
            headers = self.service_client.get_response_header()
        else:
            from openapi_client.models.bulk import (
                DownloadCampaignsByCampaignIdsRequest,
                CampaignScope
            )
            
            campaign_scopes = [
                CampaignScope(
                    campaign_id=str(campaign_id),
                    parent_account_id=str(self._authorization_data.account_id)
                )
                for campaign_id in submit_download_parameters.campaign_ids
            ]

            request = DownloadCampaignsByCampaignIdsRequest(
                campaigns=campaign_scopes,
                data_scope=DataScope.ENTITYDATA,
                download_file_type=download_file_type,
                download_entities=submit_download_parameters.download_entities,
                format_version=BULK_FORMAT_VERSION_6,
                last_sync_time_in_utc=submit_download_parameters.last_sync_time_in_utc
            )
            
            response = self.service_client.download_campaigns_by_campaign_ids(
                download_campaigns_by_campaign_ids_request=request
            )
            headers = self.service_client.get_response_header()
        operation = BulkDownloadOperation(
            request_id=response,
            authorization_data=self._authorization_data,
            poll_interval_in_milliseconds=self._poll_interval_in_milliseconds,
            environment=self._environment,
            tracking_id=headers['TrackingId'] if 'TrackingId' in headers else None,
            location=self._location,
            **self.suds_options
        )
        return operation

    def submit_upload(self, submit_upload_parameters):
        """ Submits a request for a URL where a bulk upload file may be posted.

        :param submit_upload_parameters:
        :type submit_upload_parameters: SubmitUploadParameters
        :return: The submitted upload operation.
        :rtype: BulkUploadOperation
        """

        from openapi_client.models.bulk import GetBulkUploadUrlRequest
        
        request = GetBulkUploadUrlRequest(
            account_id=self._authorization_data.account_id,
            response_mode=submit_upload_parameters.response_mode
        )
        
        response = self.service_client.get_bulk_upload_url(
            get_bulk_upload_url_request=request
        )
        headers = self.service_client.get_response_header()
        request_id = response.RequestId
        upload_url = response.UploadUrl

        if  submit_upload_parameters.rename_upload_file_to_match_request_id:
            import os
            dir = os.path.dirname(submit_upload_parameters.upload_file_path)
            new_file_to_upload = os.path.join(dir, 'upload_' + request_id + '.csv')
            os.rename(submit_upload_parameters.upload_file_path, new_file_to_upload)
            submit_upload_parameters.upload_file_path = new_file_to_upload

        self._upload_file_by_url(
            url=upload_url,
            upload_file_path=submit_upload_parameters.upload_file_path,
            compress_upload_file=submit_upload_parameters.compress_upload_file,
        )
        operation = BulkUploadOperation(
            request_id=request_id,
            authorization_data=self._authorization_data,
            poll_interval_in_milliseconds=self._poll_interval_in_milliseconds,
            environment=self._environment,
            tracking_id=headers['TrackingId'] if 'TrackingId' in headers else None,
            location=self._location,
            **self.suds_options
        )
        return operation

    def _upload_file_by_url(self, url, upload_file_path, compress_upload_file, timeout_in_milliseconds=None):
        """ Upload bulk file specified in upload parameters to specified URL

        :param url: The upload target URL.
        :type url: str
        :param upload_file_path: The fully qualified local path of the upload file.
        :type upload_file_path: str
        :param compress_upload_file: whether the upload file should be compressed before uploading.
        :type compress_upload_file: bool
        """

        _, ext = path.splitext(upload_file_path)
        if compress_upload_file and ext != '.zip':
            should_compress = True
        else:
            should_compress = False

        try:
            if should_compress:
                name, ext = path.splitext(upload_file_path)
                zip_file_path = os.path.join(self.working_directory, '{0}_{1}.zip'.format(name, uuid.uuid1()))

                with contextlib.closing(zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED)) as f:
                    f.write(upload_file_path)
                upload_file_path = zip_file_path
            headers = {
                'DeveloperToken': self._authorization_data.developer_token,
                'CustomerId': str(self._authorization_data.customer_id),
                'AccountId': str(self._authorization_data.account_id),
                'User-Agent': USER_AGENT,
            }
            self._authorization_data.authentication.enrich_headers(headers)

            with open(upload_file_path, 'rb') as f:
                name, ext = path.splitext(upload_file_path)

                filename = '{0}{1}'.format(uuid.uuid1(), ext)
                s = requests.Session()
                s.mount('https://', TlsHttpAdapter())
                timeout_seconds = None if timeout_in_milliseconds is None else timeout_in_milliseconds / 1000.0
                try:
                    r = s.post(url, files={'file': (filename, f)}, verify=True, headers=headers, timeout=timeout_seconds)
                except requests.Timeout as ex:
                    raise FileUploadException(ex)
                r.raise_for_status()
        except Exception as ex:
            raise ex
        finally:
            if should_compress:
                name, ext = path.splitext(upload_file_path)
                zip_file_path = name + '.zip'
                if path.exists(zip_file_path):
                    os.remove(zip_file_path)

    @property
    def service_client(self):
        """ The internal bulk service client.

        :rtype: ServiceClient
        """

        return self._service_client

    @property
    def poll_interval_in_milliseconds(self):
        """ The time interval in milliseconds between two status polling attempts.

        :rtype: int
        """

        return self._poll_interval_in_milliseconds

    @poll_interval_in_milliseconds.setter
    def poll_interval_in_milliseconds(self, poll_interval):
        self._poll_interval_in_milliseconds = poll_interval

    @property
    def working_directory(self):
        """ Directory for storing temporary files needed for some operations (for example :func:`upload_entities` creates a temporary upload file).

        :rtype: str
        """

        return self._working_directory

    @working_directory.setter
    def working_directory(self, value):
        self._working_directory = value

    @property
    def suds_options(self):
        """ suds option parameters

        :return: dict
        """
        return self._suds_options

    @suds_options.setter
    def suds_options(self, value):
        self._suds_options = value



# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.customer.address import Address
from openapi_client.models.customer.email_format import EmailFormat
from typing_extensions import Self

class ContactInfo(BaseModel):
    """
    ContactInfo
    """ # noqa: E501
    address: Optional[Address] = Field(default=None, alias="Address")
    contact_by_phone: Optional[StrictBool] = Field(default=None, alias="ContactByPhone")
    contact_by_postal_mail: Optional[StrictBool] = Field(default=None, alias="ContactByPostalMail")
    email: Optional[StrictStr] = Field(default=None, alias="Email")
    email_format: Optional[EmailFormat] = Field(default=None, alias="EmailFormat")
    fax: Optional[StrictStr] = Field(default=None, alias="Fax")
    home_phone: Optional[StrictStr] = Field(default=None, alias="HomePhone")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    mobile: Optional[StrictStr] = Field(default=None, alias="Mobile")
    phone1: Optional[StrictStr] = Field(default=None, alias="Phone1")
    phone2: Optional[StrictStr] = Field(default=None, alias="Phone2")
    __properties: ClassVar[List[str]] = ["Address", "ContactByPhone", "ContactByPostalMail", "Email", "EmailFormat", "Fax", "HomePhone", "Id", "Mobile", "Phone1", "Phone2"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of address
        if self.address:
            _dict['Address'] = self.address.to_dict()
        # set to None if address (nullable) is None
        # and model_fields_set contains the field
        if self.address is None and "address" in self.model_fields_set:
            _dict['Address'] = None

        # set to None if contact_by_phone (nullable) is None
        # and model_fields_set contains the field
        if self.contact_by_phone is None and "contact_by_phone" in self.model_fields_set:
            _dict['ContactByPhone'] = None

        # set to None if contact_by_postal_mail (nullable) is None
        # and model_fields_set contains the field
        if self.contact_by_postal_mail is None and "contact_by_postal_mail" in self.model_fields_set:
            _dict['ContactByPostalMail'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['Email'] = None

        # set to None if email_format (nullable) is None
        # and model_fields_set contains the field
        if self.email_format is None and "email_format" in self.model_fields_set:
            _dict['EmailFormat'] = None

        # set to None if fax (nullable) is None
        # and model_fields_set contains the field
        if self.fax is None and "fax" in self.model_fields_set:
            _dict['Fax'] = None

        # set to None if home_phone (nullable) is None
        # and model_fields_set contains the field
        if self.home_phone is None and "home_phone" in self.model_fields_set:
            _dict['HomePhone'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if mobile (nullable) is None
        # and model_fields_set contains the field
        if self.mobile is None and "mobile" in self.model_fields_set:
            _dict['Mobile'] = None

        # set to None if phone1 (nullable) is None
        # and model_fields_set contains the field
        if self.phone1 is None and "phone1" in self.model_fields_set:
            _dict['Phone1'] = None

        # set to None if phone2 (nullable) is None
        # and model_fields_set contains the field
        if self.phone2 is None and "phone2" in self.model_fields_set:
            _dict['Phone2'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ContactInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Address": Address.from_dict(obj["Address"]) if obj.get("Address") is not None else None,
                        "ContactByPhone": obj.get("ContactByPhone") if obj.get("ContactByPhone") is not None else None,
                        "ContactByPostalMail": obj.get("ContactByPostalMail") if obj.get("ContactByPostalMail") is not None else None,
                        "Email": obj.get("Email") if obj.get("Email") is not None else None,
                        "EmailFormat": obj.get("EmailFormat") if obj.get("EmailFormat") is not None else None,
                        "Fax": obj.get("Fax") if obj.get("Fax") is not None else None,
                        "HomePhone": obj.get("HomePhone") if obj.get("HomePhone") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Mobile": obj.get("Mobile") if obj.get("Mobile") is not None else None,
                        "Phone1": obj.get("Phone1") if obj.get("Phone1") is not None else None,
                        "Phone2": obj.get("Phone2") if obj.get("Phone2") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CustomerBillingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.billing.coupon_claim_info import CouponClaimInfo
from openapi_client.models.billing.coupon_redemption import CouponRedemption
from typing_extensions import Self

class Coupon(BaseModel):
    """
    Coupon
    """ # noqa: E501
    coupon_code: Optional[StrictStr] = Field(default=None, alias="CouponCode")
    class_name: Optional[StrictStr] = Field(default=None, alias="ClassName")
    coupon_type: Optional[StrictStr] = Field(default=None, alias="CouponType")
    amount: Optional[StrictFloat] = Field(default=None, alias="Amount")
    spend_threshold: Optional[StrictFloat] = Field(default=None, alias="SpendThreshold")
    currency_code: Optional[StrictStr] = Field(default=None, alias="CurrencyCode")
    percent_off: Optional[StrictFloat] = Field(default=None, alias="PercentOff")
    active_duration: Optional[StrictInt] = Field(default=None, alias="ActiveDuration")
    expiration_date: Optional[datetime] = Field(default=None, alias="ExpirationDate")
    start_date: Optional[datetime] = Field(default=None, alias="StartDate")
    end_date: Optional[datetime] = Field(default=None, alias="EndDate")
    send_to_email: Optional[StrictStr] = Field(default=None, alias="SendToEmail")
    send_to_date: Optional[datetime] = Field(default=None, alias="SendToDate")
    is_redeemed: Optional[StrictBool] = Field(default=None, alias="IsRedeemed")
    redemption_info: Optional[CouponRedemption] = Field(default=None, alias="RedemptionInfo")
    claim_info: Optional[CouponClaimInfo] = Field(default=None, alias="ClaimInfo")
    __properties: ClassVar[List[str]] = ["CouponCode", "ClassName", "CouponType", "Amount", "SpendThreshold", "CurrencyCode", "PercentOff", "ActiveDuration", "ExpirationDate", "StartDate", "EndDate", "SendToEmail", "SendToDate", "IsRedeemed", "RedemptionInfo", "ClaimInfo"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of redemption_info
        if self.redemption_info:
            _dict['RedemptionInfo'] = self.redemption_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of claim_info
        if self.claim_info:
            _dict['ClaimInfo'] = self.claim_info.to_dict()
        # set to None if coupon_code (nullable) is None
        # and model_fields_set contains the field
        if self.coupon_code is None and "coupon_code" in self.model_fields_set:
            _dict['CouponCode'] = None

        # set to None if class_name (nullable) is None
        # and model_fields_set contains the field
        if self.class_name is None and "class_name" in self.model_fields_set:
            _dict['ClassName'] = None

        # set to None if coupon_type (nullable) is None
        # and model_fields_set contains the field
        if self.coupon_type is None and "coupon_type" in self.model_fields_set:
            _dict['CouponType'] = None

        # set to None if amount (nullable) is None
        # and model_fields_set contains the field
        if self.amount is None and "amount" in self.model_fields_set:
            _dict['Amount'] = None

        # set to None if spend_threshold (nullable) is None
        # and model_fields_set contains the field
        if self.spend_threshold is None and "spend_threshold" in self.model_fields_set:
            _dict['SpendThreshold'] = None

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['CurrencyCode'] = None

        # set to None if percent_off (nullable) is None
        # and model_fields_set contains the field
        if self.percent_off is None and "percent_off" in self.model_fields_set:
            _dict['PercentOff'] = None

        # set to None if active_duration (nullable) is None
        # and model_fields_set contains the field
        if self.active_duration is None and "active_duration" in self.model_fields_set:
            _dict['ActiveDuration'] = None

        # set to None if expiration_date (nullable) is None
        # and model_fields_set contains the field
        if self.expiration_date is None and "expiration_date" in self.model_fields_set:
            _dict['ExpirationDate'] = None

        # set to None if start_date (nullable) is None
        # and model_fields_set contains the field
        if self.start_date is None and "start_date" in self.model_fields_set:
            _dict['StartDate'] = None

        # set to None if end_date (nullable) is None
        # and model_fields_set contains the field
        if self.end_date is None and "end_date" in self.model_fields_set:
            _dict['EndDate'] = None

        # set to None if send_to_email (nullable) is None
        # and model_fields_set contains the field
        if self.send_to_email is None and "send_to_email" in self.model_fields_set:
            _dict['SendToEmail'] = None

        # set to None if send_to_date (nullable) is None
        # and model_fields_set contains the field
        if self.send_to_date is None and "send_to_date" in self.model_fields_set:
            _dict['SendToDate'] = None

        # set to None if is_redeemed (nullable) is None
        # and model_fields_set contains the field
        if self.is_redeemed is None and "is_redeemed" in self.model_fields_set:
            _dict['IsRedeemed'] = None

        # set to None if redemption_info (nullable) is None
        # and model_fields_set contains the field
        if self.redemption_info is None and "redemption_info" in self.model_fields_set:
            _dict['RedemptionInfo'] = None

        # set to None if claim_info (nullable) is None
        # and model_fields_set contains the field
        if self.claim_info is None and "claim_info" in self.model_fields_set:
            _dict['ClaimInfo'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Coupon from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CouponCode": obj.get("CouponCode") if obj.get("CouponCode") is not None else None,
                        "ClassName": obj.get("ClassName") if obj.get("ClassName") is not None else None,
                        "CouponType": obj.get("CouponType") if obj.get("CouponType") is not None else None,
                        "Amount": obj.get("Amount") if obj.get("Amount") is not None else None,
                        "SpendThreshold": obj.get("SpendThreshold") if obj.get("SpendThreshold") is not None else None,
                        "CurrencyCode": obj.get("CurrencyCode") if obj.get("CurrencyCode") is not None else None,
                        "PercentOff": obj.get("PercentOff") if obj.get("PercentOff") is not None else None,
                        "ActiveDuration": obj.get("ActiveDuration") if obj.get("ActiveDuration") is not None else None,
                        "ExpirationDate": obj.get("ExpirationDate") if obj.get("ExpirationDate") is not None else None,
                        "StartDate": obj.get("StartDate") if obj.get("StartDate") is not None else None,
                        "EndDate": obj.get("EndDate") if obj.get("EndDate") is not None else None,
                        "SendToEmail": obj.get("SendToEmail") if obj.get("SendToEmail") is not None else None,
                        "SendToDate": obj.get("SendToDate") if obj.get("SendToDate") is not None else None,
                        "IsRedeemed": obj.get("IsRedeemed") if obj.get("IsRedeemed") is not None else None,
                        "RedemptionInfo": CouponRedemption.from_dict(obj["RedemptionInfo"]) if obj.get("RedemptionInfo") is not None else None,
                        "ClaimInfo": CouponClaimInfo.from_dict(obj["ClaimInfo"]) if obj.get("ClaimInfo") is not None else None
        })
        return _obj



from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity


class UnknownBulkEntity(_SingleRecordBulkEntity):
    """ Reserved to support new record types that may be added to the Bulk schema. """

    def __init__(self):
        super(UnknownBulkEntity, self).__init__()
        self._values = None

    @property
    def values(self):
        """ The forward compatibility map of fields and values.

        :rtype: dict | None
        """

        return self._values

    def process_mappings_from_row_values(self, row_values):
        self._values = row_values.to_dict()

    def process_mappings_to_row_values(self, row_values):
        for (key, value) in self._values.items():
            row_values[key] = value

    def read_additional_data(self, stream_reader):
        super(UnknownBulkEntity, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class BroadMatchSearchQueryKPI(BaseModel):
    """
    BroadMatchSearchQueryKPI
    """ # noqa: E501
    search_query: Optional[StrictStr] = Field(default=None, alias="SearchQuery")
    impressions: Optional[StrictStr] = Field(default=None, alias="Impressions")
    clicks: Optional[StrictFloat] = Field(default=None, alias="Clicks")
    srpv: Optional[StrictStr] = Field(default=None, alias="SRPV")
    average_ctr: Optional[StrictFloat] = Field(default=None, alias="AverageCTR")
    __properties: ClassVar[List[str]] = ["SearchQuery", "Impressions", "Clicks", "SRPV", "AverageCTR"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if search_query (nullable) is None
        # and model_fields_set contains the field
        if self.search_query is None and "search_query" in self.model_fields_set:
            _dict['SearchQuery'] = None

        # set to None if impressions (nullable) is None
        # and model_fields_set contains the field
        if self.impressions is None and "impressions" in self.model_fields_set:
            _dict['Impressions'] = None

        # set to None if clicks (nullable) is None
        # and model_fields_set contains the field
        if self.clicks is None and "clicks" in self.model_fields_set:
            _dict['Clicks'] = None

        # set to None if srpv (nullable) is None
        # and model_fields_set contains the field
        if self.srpv is None and "srpv" in self.model_fields_set:
            _dict['SRPV'] = None

        # set to None if average_ctr (nullable) is None
        # and model_fields_set contains the field
        if self.average_ctr is None and "average_ctr" in self.model_fields_set:
            _dict['AverageCTR'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BroadMatchSearchQueryKPI from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "SearchQuery": obj.get("SearchQuery") if obj.get("SearchQuery") is not None else None,
                        "Impressions": obj.get("Impressions") if obj.get("Impressions") is not None else None,
                        "Clicks": obj.get("Clicks") if obj.get("Clicks") is not None else None,
                        "SRPV": obj.get("SRPV") if obj.get("SRPV") is not None else None,
                        "AverageCTR": obj.get("AverageCTR") if obj.get("AverageCTR") is not None else None
        })
        return _obj



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

class BulkAssetGroupSearchTheme(_SingleRecordBulkEntity):
    """ Represents an asset group search theme.

    This class exposes the property :attr:`asset_group_search_theme` that can be read and written as fields of the asset group search theme record
    in a bulk file.

    For more information, see asset group search theme at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, asset_group_id=None, asset_group_search_theme=None):
        super(BulkAssetGroupSearchTheme, self).__init__()

        self._asset_group_search_theme = asset_group_search_theme
        self._asset_group_id = asset_group_id

    @property
    def asset_group_search_theme(self):
        """ The AssetGroupSearchTheme Data Object of the Campaign Management Service.

        A subset of AssetGroupSearchTheme properties are available in the Ad Group record.
        For more information, see Ad Group at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._asset_group_search_theme

    @asset_group_search_theme.setter
    def asset_group_search_theme(self, asset_group_search_theme):
        self._asset_group_search_theme = asset_group_search_theme

    @property
    def asset_group_id(self):
        """ The identifier of the asset group that contains the search theme.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._asset_group_id

    @asset_group_id.setter
    def asset_group_id(self, asset_group_id):
        self._asset_group_id = asset_group_id


    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.asset_group_search_theme.Id),
            csv_to_field=lambda c, v: setattr(c.asset_group_search_theme, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.asset_group_id),
            csv_to_field=lambda c, v: setattr(c, 'asset_group_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SearchTheme,
            field_to_csv=lambda c: bulk_str(c.asset_group_search_theme.SearchTheme),
            csv_to_field=lambda c, v: setattr(c.asset_group_search_theme, 'SearchTheme', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.asset_group_search_theme = _CAMPAIGN_OBJECT_FACTORY_V13.create('AssetGroupSearchTheme')

        row_values.convert_to_entity(self, BulkAssetGroupSearchTheme._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._asset_group_search_theme, 'AssetGroupSearchTheme')
        self.convert_to_values(row_values, BulkAssetGroupSearchTheme._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAssetGroupSearchTheme, self).read_additional_data(stream_reader)



# coding: utf-8

# flake8: noqa
"""
    CustomerBillingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.billing.account_additional_field import AccountAdditionalField
from openapi_client.models.billing.ad_api_error import AdApiError
from openapi_client.models.billing.ad_api_fault_detail import AdApiFaultDetail
from openapi_client.models.billing.add_insertion_order_request import AddInsertionOrderRequest
from openapi_client.models.billing.add_insertion_order_response import AddInsertionOrderResponse
from openapi_client.models.billing.api_batch_fault import ApiBatchFault
from openapi_client.models.billing.api_fault import ApiFault
from openapi_client.models.billing.application_fault import ApplicationFault
from openapi_client.models.billing.batch_error import BatchError
from openapi_client.models.billing.billing_document import BillingDocument
from openapi_client.models.billing.billing_document_info import BillingDocumentInfo
from openapi_client.models.billing.check_feature_adoption_coupon_eligibility_request import CheckFeatureAdoptionCouponEligibilityRequest
from openapi_client.models.billing.check_feature_adoption_coupon_eligibility_response import CheckFeatureAdoptionCouponEligibilityResponse
from openapi_client.models.billing.claim_feature_adoption_coupons_request import ClaimFeatureAdoptionCouponsRequest
from openapi_client.models.billing.claim_feature_adoption_coupons_response import ClaimFeatureAdoptionCouponsResponse
from openapi_client.models.billing.coupon import Coupon
from openapi_client.models.billing.coupon_claim_info import CouponClaimInfo
from openapi_client.models.billing.coupon_info_data import CouponInfoData
from openapi_client.models.billing.coupon_redemption import CouponRedemption
from openapi_client.models.billing.data_type import DataType
from openapi_client.models.billing.dispatch_coupons_request import DispatchCouponsRequest
from openapi_client.models.billing.dispatch_coupons_response import DispatchCouponsResponse
from openapi_client.models.billing.get_account_monthly_spend_request import GetAccountMonthlySpendRequest
from openapi_client.models.billing.get_account_monthly_spend_response import GetAccountMonthlySpendResponse
from openapi_client.models.billing.get_billing_documents_info_request import GetBillingDocumentsInfoRequest
from openapi_client.models.billing.get_billing_documents_info_response import GetBillingDocumentsInfoResponse
from openapi_client.models.billing.get_billing_documents_request import GetBillingDocumentsRequest
from openapi_client.models.billing.get_billing_documents_response import GetBillingDocumentsResponse
from openapi_client.models.billing.get_coupon_info_request import GetCouponInfoRequest
from openapi_client.models.billing.get_coupon_info_response import GetCouponInfoResponse
from openapi_client.models.billing.insertion_order import InsertionOrder
from openapi_client.models.billing.insertion_order_additional_field import InsertionOrderAdditionalField
from openapi_client.models.billing.insertion_order_pending_changes import InsertionOrderPendingChanges
from openapi_client.models.billing.insertion_order_pending_changes_status import InsertionOrderPendingChangesStatus
from openapi_client.models.billing.insertion_order_status import InsertionOrderStatus
from openapi_client.models.billing.key_value_entity_oflong_and_date_time import KeyValueEntityOflongAndDateTime
from openapi_client.models.billing.key_value_entity_oflong_andstring import KeyValueEntityOflongAndstring
from openapi_client.models.billing.operation_error import OperationError
from openapi_client.models.billing.order_by import OrderBy
from openapi_client.models.billing.order_by_field import OrderByField
from openapi_client.models.billing.paging import Paging
from openapi_client.models.billing.predicate import Predicate
from openapi_client.models.billing.predicate_operator import PredicateOperator
from openapi_client.models.billing.redeem_coupon_request import RedeemCouponRequest
from openapi_client.models.billing.redeem_coupon_response import RedeemCouponResponse
from openapi_client.models.billing.search_coupons_request import SearchCouponsRequest
from openapi_client.models.billing.search_coupons_response import SearchCouponsResponse
from openapi_client.models.billing.search_insertion_orders_request import SearchInsertionOrdersRequest
from openapi_client.models.billing.search_insertion_orders_response import SearchInsertionOrdersResponse
from openapi_client.models.billing.sort_order import SortOrder
from openapi_client.models.billing.update_insertion_order_request import UpdateInsertionOrderRequest
from openapi_client.models.billing.update_insertion_order_response import UpdateInsertionOrderResponse



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.bulk.entities.target_criterions.bulk_ad_group_negative_criterion import BulkAdGroupNegativeCriterion
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from abc import ABCMeta, abstractmethod


class BulkAdGroupNegativeProfileCriterion(BulkAdGroupNegativeCriterion):
    """ Represents an Ad Group Negative Profile Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`negative_ad_group_criterion` property that can be read and written as fields of the
    Ad Group Negative Profile Criterion record in a bulk file.

    For more information, see Ad Group Negative Profile Criterion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManProfiler`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 negative_ad_group_criterion=None,
                 campaign_name=None,
                 ad_group_name=None, ):
        super(BulkAdGroupNegativeProfileCriterion, self).__init__(negative_ad_group_criterion, campaign_name, ad_group_name)

    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.Profile,
            field_to_csv=lambda c: c.profile_name,
            csv_to_field=lambda c, v: setattr(c, 'profile_name', v)
        ),
        _SimpleBulkMapping(
            _StringTable.ProfileId,
            field_to_csv=lambda c: bulk_str(c.negative_ad_group_criterion.Criterion.ProfileId),
            csv_to_field=lambda c, v: setattr(c.negative_ad_group_criterion.Criterion, 'ProfileId', int(v) if v else None)
        ),
    ]

    def create_criterion(self):
        self._negative_ad_group_criterion.Criterion = _CAMPAIGN_OBJECT_FACTORY_V13.create('ProfileCriterion')
        self._negative_ad_group_criterion.Criterion.Type = 'ProfileCriterion'
        self._negative_ad_group_criterion.Criterion.ProfileType = self.profile_type()

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(BulkAdGroupNegativeProfileCriterion, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkAdGroupNegativeProfileCriterion._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        super(BulkAdGroupNegativeProfileCriterion, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkAdGroupNegativeProfileCriterion._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAdGroupNegativeProfileCriterion, self).read_additional_data(stream_reader)

    @abstractmethod
    def profile_type(self):
        pass

class BulkAdGroupNegativeCompanyNameCriterion(BulkAdGroupNegativeProfileCriterion):
    """ Represents an Ad Group Negative CompanyName Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`negative_ad_group_criterion` property that can be read and written as fields of the
    Ad Group Negative CompanyName Criterion record in a bulk file.

    For more information, see Ad Group Negative CompanyName Criterion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManCompanyNamer`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 negative_ad_group_criterion=None,
                 campaign_name=None,
                 ad_group_name=None, ):
        super(BulkAdGroupNegativeCompanyNameCriterion, self).__init__()

        self._negative_ad_group_criterion = negative_ad_group_criterion
        self._campaign_name = campaign_name
        self._ad_group_name = ad_group_name
        
    def profile_type(self):
        return 'CompanyName'
    
    
class BulkAdGroupNegativeIndustryCriterion(BulkAdGroupNegativeProfileCriterion):
    """ Represents an Ad Group Negative Industry Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`negative_ad_group_criterion` property that can be read and written as fields of the
    Ad Group Negative Industry Criterion record in a bulk file.

    For more information, see Ad Group Negative Industry Criterion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManIndustryr`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 negative_ad_group_criterion=None,
                 campaign_name=None,
                 ad_group_name=None, ):
        super(BulkAdGroupNegativeIndustryCriterion, self).__init__()

        self._negative_ad_group_criterion = negative_ad_group_criterion
        self._campaign_name = campaign_name
        self._ad_group_name = ad_group_name
 
    def profile_type(self):
        return 'Industry'
    
    
class BulkAdGroupNegativeJobFunctionCriterion(BulkAdGroupNegativeProfileCriterion):
    """ Represents an Ad Group Negative JobFunction Criterion that can be read or written in a bulk file.

    This class exposes the :attr:`negative_ad_group_criterion` property that can be read and written as fields of the
    Ad Group Negative JobFunction Criterion record in a bulk file.

    For more information, see Ad Group Negative JobFunction Criterion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManJobFunctionr`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 negative_ad_group_criterion=None,
                 campaign_name=None,
                 ad_group_name=None, ):
        super(BulkAdGroupNegativeJobFunctionCriterion, self).__init__()

        self._negative_ad_group_criterion = negative_ad_group_criterion
        self._campaign_name = campaign_name
        self._ad_group_name = ad_group_name

    def profile_type(self):
        return 'JobFunction'


# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AgeGenderAudienceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    TIMEPERIOD = 'TimePeriod'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    ADDISTRIBUTION = 'AdDistribution'

    AGEGROUP = 'AgeGroup'

    GENDER = 'Gender'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CONVERSIONS = 'Conversions'

    SPEND = 'Spend'

    REVENUE = 'Revenue'

    EXTENDEDCOST = 'ExtendedCost'

    ASSISTS = 'Assists'

    LANGUAGE = 'Language'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    BASECAMPAIGNID = 'BaseCampaignId'

    ALLCONVERSIONS = 'AllConversions'

    ALLREVENUE = 'AllRevenue'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    ABSOLUTETOPIMPRESSIONRATEPERCENT = 'AbsoluteTopImpressionRatePercent'

    TOPIMPRESSIONRATEPERCENT = 'TopImpressionRatePercent'

    CONVERSIONSQUALIFIED = 'ConversionsQualified'

    ALLCONVERSIONSQUALIFIED = 'AllConversionsQualified'

    VIEWTHROUGHCONVERSIONSQUALIFIED = 'ViewThroughConversionsQualified'

    VIEWTHROUGHREVENUE = 'ViewThroughRevenue'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AgeGenderAudienceReportColumn from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *


class _BulkLabelAssociation(_SingleRecordBulkEntity):
    """ Represents a label association that can be read or written in a bulk file.

    This class exposes the :attr:`label_association` property that can be read and written as fields of the Keyword record in a bulk file.
    Properties of this class and of classes that it is derived from, correspond to fields of the Keyword record in a bulk file.
    For more information, see Keyword at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(_BulkLabelAssociation, self).__init__()
        self._label_association = label_association
        self._status = status

    @property
    def label_association(self):
        """ The LabelAssociation Data Object of the Campaign Management Service.

        A subset of Label properties are available in the Ad Group record.
        """

        return self._label_association

    @label_association.setter
    def label_association(self, value):
        self._label_association = value

    @property
    def status(self):
        """ the status of bulk record
        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(
                c,
                'status',
                v if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: c.label_association.LabelId,
            csv_to_field=lambda c, v: setattr(
                c.label_association,
                'LabelId',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: c.label_association.EntityId,
            csv_to_field=lambda c, v: setattr(
                c.label_association,
                'EntityId',
                int(v) if v else None
            )
        ),

    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._label_association, 'label_association')
        self.convert_to_values(row_values, _BulkLabelAssociation._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self._label_association = _CAMPAIGN_OBJECT_FACTORY_V13.create('LabelAssociation')
        row_values.convert_to_entity(self, _BulkLabelAssociation._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(_BulkLabelAssociation, self).read_additional_data(stream_reader)


class BulkCampaignLabel(_BulkLabelAssociation):
    """ Represents a campaign label.

    Defines an association record between a Campaign and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see Campaign Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None, campaign=None):
        super(BulkCampaignLabel, self).__init__(label_association, status)
        self._campaign = campaign

    @property
    def campaign(self):
        """ The campaign name of the Campaign Management Service.

        A subset of Label properties are available in the Ad Group record.
        """

        return self._campaign

    @campaign.setter
    def campaign(self, value):
        self._campaign = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign,
            csv_to_field=lambda c, v: setattr(
                c,
                'campaign',
                v
            )
        )
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(BulkCampaignLabel, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkCampaignLabel._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        super(BulkCampaignLabel, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkCampaignLabel._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkCampaignLabel, self).read_additional_data(stream_reader)


class BulkAdGroupLabel(_BulkLabelAssociation):
    """ Represents a AdGroup label.

    Defines an association record between a AdGroup and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see AdGroup Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None, campaign=None, ad_group=None):
        super(BulkAdGroupLabel, self).__init__(label_association, status)
        self._campaign = campaign
        self._ad_group = ad_group

    @property
    def campaign(self):
        """ The campaign name of the Campaign Management Service.

        A subset of Label properties are available in the Ad Group record.
        """

        return self._campaign

    @campaign.setter
    def campaign(self, value):
        self._campaign = value

    @property
    def ad_group(self):
        """ The ad group name of the Campaign Management Service.

        A subset of Label properties are available in the Ad Group record.
        """

        return self._campaign

    @ad_group.setter
    def ad_group(self, value):
        self._ad_group = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign,
            csv_to_field=lambda c, v: setattr(
                c,
                'campaign',
                v
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdGroup,
            field_to_csv=lambda c: c.ad_group,
            csv_to_field=lambda c, v: setattr(
                c,
                'ad_group',
                v
            )
        )
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(BulkAdGroupLabel, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkAdGroupLabel._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        super(BulkAdGroupLabel, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkAdGroupLabel._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAdGroupLabel, self).read_additional_data(stream_reader)


class BulkKeywordLabel(_BulkLabelAssociation):
    """ Represents a Keyword label.

    Defines an association record between a Keyword and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see Keyword Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkKeywordLabel, self).__init__(label_association, status)


class BulkAppInstallAdLabel(_BulkLabelAssociation):
    """ Represents a AppInstallAd label.

    Defines an association record between a AppInstallAd and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see AppInstallAd Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkAppInstallAdLabel, self).__init__(label_association, status)


class BulkDynamicSearchAdLabel(_BulkLabelAssociation):
    """ Represents a DynamicSearchAd label.

    Defines an association record between a DynamicSearchAd and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see DynamicSearchAd Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkDynamicSearchAdLabel, self).__init__(label_association, status)


class BulkExpandedTextAdLabel(_BulkLabelAssociation):
    """ Represents a ExpandedTextAd label.

    Defines an association record between a ExpandedTextAd and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see ExpandedTextAd Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkExpandedTextAdLabel, self).__init__(label_association, status)


class BulkProductAdLabel(_BulkLabelAssociation):
    """ Represents a ProductAd label.

    Defines an association record between a ProductAd and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see ProductAd Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkProductAdLabel, self).__init__(label_association, status)


class BulkTextAdLabel(_BulkLabelAssociation):
    """ Represents a TextAd label.

    Defines an association record between a TextAd and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see TextAd Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkTextAdLabel, self).__init__(label_association, status)

class BulkResponsiveAdLabel(_BulkLabelAssociation):
    """ Represents a ResponsiveAd label.

    Defines an association record between a ResponsiveAd and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see ResponsiveAd Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkResponsiveAdLabel, self).__init__(label_association, status)

class BulkResponsiveSearchAdLabel(_BulkLabelAssociation):
    """ Represents a ResponsiveSearchAd label.

    Defines an association record between a ResponsiveSearchAd and a Label that can be uploaded and downloaded in a bulk file.

    For more information, see ResponsiveSearchAd Label at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, label_association=None, status=None):
        super(BulkResponsiveSearchAdLabel, self).__init__(label_association, status)


from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *

class BulkAssetGroupListingGroup(_SingleRecordBulkEntity):
    """ Represents an asset group listing group.

    This class exposes the property :attr:`asset_group_listing_group` that can be read and written as fields of the Asset Group Listing Group record
    in a bulk file.

    For more information, see Asset Group at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None, asset_group_listing_group=None):
        super(BulkAssetGroupListingGroup, self).__init__()

        self._status = status
        self._asset_group_listing_group = asset_group_listing_group
        self._asset_group_name = None
        self._campaign_name = None

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def asset_group_name(self):
        return self._asset_group_name

    @asset_group_name.setter
    def asset_group_name(self, asset_group_name):
        self._asset_group_name = asset_group_name

    @property
    def campaign_name(self):
        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def asset_group_listing_group(self):
        """ The AssetGroupListingGroup Data Object of the Campaign Management Service.

        A subset of AssetGroupListingGroup properties are available in the Ad Group record.
        For more information, see Ad Group at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._asset_group_listing_group

    @asset_group_listing_group.setter
    def asset_group_listing_group(self, asset_group_listing_group):
        self._asset_group_listing_group = asset_group_listing_group

    @classmethod
    def _get_condition_operand(cls, entity):
        if entity.asset_group_listing_group is not None and \
                hasattr(entity.asset_group_listing_group, 'Dimension') and \
                entity.asset_group_listing_group.Dimension is not None and \
                hasattr(entity.asset_group_listing_group.Dimension, 'Operand'):
            return entity.asset_group_listing_group.Dimension.Operand
        return None

    @classmethod
    def _get_condition_attribute(cls, entity):
        if entity.asset_group_listing_group is not None and \
                hasattr(entity.asset_group_listing_group, 'Dimension') and \
                entity.asset_group_listing_group.Dimension is not None and \
                hasattr(entity.asset_group_listing_group.Dimension, 'Attribute'):
            return entity.asset_group_listing_group.Dimension.Attribute
        return None

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.asset_group_listing_group.Id),
            csv_to_field=lambda c, v: setattr(c.asset_group_listing_group, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.asset_group_listing_group.AssetGroupId),
            csv_to_field=lambda c, v: setattr(c.asset_group_listing_group, 'AssetGroupId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroup,
            field_to_csv=lambda c: c.asset_group_name,
            csv_to_field=lambda c, v: setattr(c, 'asset_group_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, 'campaign_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.IsExcluded,
            field_to_csv=lambda c: bulk_str(c.asset_group_listing_group.IsExcluded),
            csv_to_field=lambda c, v: setattr(c.asset_group_listing_group, 'IsExcluded', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentListingGroupId,
            field_to_csv=lambda c: bulk_str(c.asset_group_listing_group.ParentListingGroupId),
            csv_to_field=lambda c, v: setattr(c.asset_group_listing_group, 'ParentListingGroupId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SubType,
            field_to_csv=lambda c: bulk_str(c.asset_group_listing_group.AssetGroupListingType),
            csv_to_field=lambda c, v: setattr(c.asset_group_listing_group, 'AssetGroupListingType', v if v else None)
        ),
        _SimpleBulkMapping(
            _StringTable.ProductCondition1,
            field_to_csv=lambda c: BulkAssetGroupListingGroup._get_condition_operand(c),
            csv_to_field=lambda c, v: setattr(c.asset_group_listing_group.Dimension, 'Operand', v)
        ),
        _SimpleBulkMapping(
            _StringTable.ProductValue1,
            field_to_csv=lambda c: BulkAssetGroupListingGroup._get_condition_attribute(c),
            csv_to_field=lambda c, v: setattr(c.asset_group_listing_group.Dimension, 'Attribute', v)
        ),
    ]


    def process_mappings_from_row_values(self, row_values):
        self.asset_group_listing_group = _CAMPAIGN_OBJECT_FACTORY_V13.create('AssetGroupListingGroup')

        row_values.convert_to_entity(self, BulkAssetGroupListingGroup._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._asset_group_listing_group, 'AssetGroupListingGroup')
        self.convert_to_values(row_values, BulkAssetGroupListingGroup._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAssetGroupListingGroup, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class ImageAsset(BaseModel):
    """
    ImageAsset
    """ # noqa: E501
    sub_type: Optional[StrictStr] = Field(default=None, alias="SubType")
    crop_x: Optional[StrictInt] = Field(default=None, alias="CropX")
    crop_y: Optional[StrictInt] = Field(default=None, alias="CropY")
    crop_width: Optional[StrictInt] = Field(default=None, alias="CropWidth")
    crop_height: Optional[StrictInt] = Field(default=None, alias="CropHeight")
    target_width: Optional[StrictInt] = Field(default=None, alias="TargetWidth")
    target_height: Optional[StrictInt] = Field(default=None, alias="TargetHeight")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    type: Optional[StrictStr] = Field(default='ImageAsset', alias="Type")
    __properties: ClassVar[List[str]] = ["SubType", "CropX", "CropY", "CropWidth", "CropHeight", "TargetWidth", "TargetHeight", "Id", "Name", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ImageAsset from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if sub_type (nullable) is None
        # and model_fields_set contains the field
        if self.sub_type is None and "sub_type" in self.model_fields_set:
            _dict['SubType'] = None

        # set to None if crop_x (nullable) is None
        # and model_fields_set contains the field
        if self.crop_x is None and "crop_x" in self.model_fields_set:
            _dict['CropX'] = None

        # set to None if crop_y (nullable) is None
        # and model_fields_set contains the field
        if self.crop_y is None and "crop_y" in self.model_fields_set:
            _dict['CropY'] = None

        # set to None if crop_width (nullable) is None
        # and model_fields_set contains the field
        if self.crop_width is None and "crop_width" in self.model_fields_set:
            _dict['CropWidth'] = None

        # set to None if crop_height (nullable) is None
        # and model_fields_set contains the field
        if self.crop_height is None and "crop_height" in self.model_fields_set:
            _dict['CropHeight'] = None

        # set to None if target_width (nullable) is None
        # and model_fields_set contains the field
        if self.target_width is None and "target_width" in self.model_fields_set:
            _dict['TargetWidth'] = None

        # set to None if target_height (nullable) is None
        # and model_fields_set contains the field
        if self.target_height is None and "target_height" in self.model_fields_set:
            _dict['TargetHeight'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ImageAsset from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "SubType": obj.get("SubType") if obj.get("SubType") is not None else None,
                        "CropX": obj.get("CropX") if obj.get("CropX") is not None else None,
                        "CropY": obj.get("CropY") if obj.get("CropY") is not None else None,
                        "CropWidth": obj.get("CropWidth") if obj.get("CropWidth") is not None else None,
                        "CropHeight": obj.get("CropHeight") if obj.get("CropHeight") is not None else None,
                        "TargetWidth": obj.get("TargetWidth") if obj.get("TargetWidth") is not None else None,
                        "TargetHeight": obj.get("TargetHeight") if obj.get("TargetHeight") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'ImageAsset'
        })
        return _obj



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

class BulkBrandList(_SingleRecordBulkEntity):
    """ Represents a campaign brand list association.

    This class exposes the property :attr:`campaign_brand_list_association` that can be read and written as fields of the campaign brand list association record
    in a bulk file.

    For more information, see campaign brand list association at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, brand_list=None):
        super(BulkBrandList, self).__init__()

        self._account_id = account_id
        self._brand_list = brand_list

    @property
    def brand_list(self):
        """ The BrandList Data Object of the Campaign Management Service.

        A subset of BrandList properties are available in the Brand List record.
        For more information, see Brand Item at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._brand_list

    @brand_list.setter
    def brand_list(self, brand_list):
        self._brand_list = brand_list

    @property
    def account_id(self):
        """ The identifier of the account that contains the brand list.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        self._account_id = account_id


    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.brand_list.Id),
            csv_to_field=lambda c, v: setattr(c.brand_list, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: bulk_str(c.brand_list.Name),
            csv_to_field=lambda c, v: setattr(c.brand_list, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, 'account_id', int(v) if v else None)
        ),

    ]

    def process_mappings_from_row_values(self, row_values):
        self.brand_list = _CAMPAIGN_OBJECT_FACTORY_V13.create('BrandList')

        row_values.convert_to_entity(self, BulkBrandList._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.brand_list, 'BrandList')
        self.convert_to_values(row_values, BulkBrandList._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkBrandList, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.bid_landscape_point import BidLandscapePoint
from openapi_client.models.adinsight.day_month_and_year import DayMonthAndYear
from typing_extensions import Self

class KeywordBidLandscape(BaseModel):
    """
    KeywordBidLandscape
    """ # noqa: E501
    keyword_id: Optional[StrictStr] = Field(default=None, alias="KeywordId")
    start_date: Optional[DayMonthAndYear] = Field(default=None, alias="StartDate")
    end_date: Optional[DayMonthAndYear] = Field(default=None, alias="EndDate")
    bid_landscape_points: Optional[List[Optional[BidLandscapePoint]]] = Field(default=None, alias="BidLandscapePoints")
    __properties: ClassVar[List[str]] = ["KeywordId", "StartDate", "EndDate", "BidLandscapePoints"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of start_date
        if self.start_date:
            _dict['StartDate'] = self.start_date.to_dict()
        # override the default output from pydantic by calling `to_dict()` of end_date
        if self.end_date:
            _dict['EndDate'] = self.end_date.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in bid_landscape_points (list)
        _items = []
        if self.bid_landscape_points:
            for _item_bid_landscape_points in self.bid_landscape_points:
                if _item_bid_landscape_points:
                    _items.append(_item_bid_landscape_points.to_dict())
            _dict['BidLandscapePoints'] = _items
        # set to None if keyword_id (nullable) is None
        # and model_fields_set contains the field
        if self.keyword_id is None and "keyword_id" in self.model_fields_set:
            _dict['KeywordId'] = None

        # set to None if start_date (nullable) is None
        # and model_fields_set contains the field
        if self.start_date is None and "start_date" in self.model_fields_set:
            _dict['StartDate'] = None

        # set to None if end_date (nullable) is None
        # and model_fields_set contains the field
        if self.end_date is None and "end_date" in self.model_fields_set:
            _dict['EndDate'] = None

        # set to None if bid_landscape_points (nullable) is None
        # and model_fields_set contains the field
        if self.bid_landscape_points is None and "bid_landscape_points" in self.model_fields_set:
            _dict['BidLandscapePoints'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of KeywordBidLandscape from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "KeywordId": obj.get("KeywordId") if obj.get("KeywordId") is not None else None,
                        "StartDate": DayMonthAndYear.from_dict(obj["StartDate"]) if obj.get("StartDate") is not None else None,
                        "EndDate": DayMonthAndYear.from_dict(obj["EndDate"]) if obj.get("EndDate") is not None else None,
                        "BidLandscapePoints": [BidLandscapePoint.from_dict(_item) for _item in obj["BidLandscapePoints"]] if obj.get("BidLandscapePoints") is not None else None
        })
        return _obj



# Import all models from campaign module
from openapi_client.models.campaign import *
from openapi_client.models.bulk import *
from openapi_client.models.reporting import *
from openapi_client.models.customer import *
from openapi_client.models.billing import *
from openapi_client.models.adinsight import *
# Import other models as needed


# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.app_url import AppUrl
from openapi_client.models.campaign.bid import Bid
from openapi_client.models.campaign.bidding_scheme import BiddingScheme
from openapi_client.models.campaign.custom_parameters import CustomParameters
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.keyword_editorial_status import KeywordEditorialStatus
from openapi_client.models.campaign.keyword_status import KeywordStatus
from openapi_client.models.campaign.match_type import MatchType
from typing_extensions import Self

class Keyword(BaseModel):
    """
    Keyword
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    bid: Optional[Bid] = Field(default=None, alias="Bid")
    match_type: Optional[MatchType] = Field(default=None, alias="MatchType")
    param1: Optional[StrictStr] = Field(default=None, alias="Param1")
    param2: Optional[StrictStr] = Field(default=None, alias="Param2")
    param3: Optional[StrictStr] = Field(default=None, alias="Param3")
    text: Optional[StrictStr] = Field(default=None, alias="Text")
    status: Optional[KeywordStatus] = Field(default=None, alias="Status")
    editorial_status: Optional[KeywordEditorialStatus] = Field(default=None, alias="EditorialStatus")
    destination_url: Optional[StrictStr] = Field(default=None, alias="DestinationUrl")
    tracking_url_template: Optional[StrictStr] = Field(default=None, alias="TrackingUrlTemplate")
    final_url_suffix: Optional[StrictStr] = Field(default=None, alias="FinalUrlSuffix")
    url_custom_parameters: Optional[CustomParameters] = Field(default=None, alias="UrlCustomParameters")
    final_urls: Optional[List[StrictStr]] = Field(default=None, alias="FinalUrls")
    final_mobile_urls: Optional[List[StrictStr]] = Field(default=None, alias="FinalMobileUrls")
    final_app_urls: Optional[List[Optional[AppUrl]]] = Field(default=None, alias="FinalAppUrls")
    bidding_scheme: Optional[BiddingScheme] = Field(default=None, alias="BiddingScheme")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    __properties: ClassVar[List[str]] = ["Id", "Bid", "MatchType", "Param1", "Param2", "Param3", "Text", "Status", "EditorialStatus", "DestinationUrl", "TrackingUrlTemplate", "FinalUrlSuffix", "UrlCustomParameters", "FinalUrls", "FinalMobileUrls", "FinalAppUrls", "BiddingScheme", "ForwardCompatibilityMap"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of bid
        if self.bid:
            _dict['Bid'] = self.bid.to_dict()
        # override the default output from pydantic by calling `to_dict()` of url_custom_parameters
        if self.url_custom_parameters:
            _dict['UrlCustomParameters'] = self.url_custom_parameters.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in final_app_urls (list)
        _items = []
        if self.final_app_urls:
            for _item_final_app_urls in self.final_app_urls:
                if _item_final_app_urls:
                    _items.append(_item_final_app_urls.to_dict())
            _dict['FinalAppUrls'] = _items
        # override the default output from pydantic by calling `to_dict()` of bidding_scheme
        if self.bidding_scheme:
            _dict['BiddingScheme'] = self.bidding_scheme.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if bid (nullable) is None
        # and model_fields_set contains the field
        if self.bid is None and "bid" in self.model_fields_set:
            _dict['Bid'] = None

        # set to None if match_type (nullable) is None
        # and model_fields_set contains the field
        if self.match_type is None and "match_type" in self.model_fields_set:
            _dict['MatchType'] = None

        # set to None if param1 (nullable) is None
        # and model_fields_set contains the field
        if self.param1 is None and "param1" in self.model_fields_set:
            _dict['Param1'] = None

        # set to None if param2 (nullable) is None
        # and model_fields_set contains the field
        if self.param2 is None and "param2" in self.model_fields_set:
            _dict['Param2'] = None

        # set to None if param3 (nullable) is None
        # and model_fields_set contains the field
        if self.param3 is None and "param3" in self.model_fields_set:
            _dict['Param3'] = None

        # set to None if text (nullable) is None
        # and model_fields_set contains the field
        if self.text is None and "text" in self.model_fields_set:
            _dict['Text'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if editorial_status (nullable) is None
        # and model_fields_set contains the field
        if self.editorial_status is None and "editorial_status" in self.model_fields_set:
            _dict['EditorialStatus'] = None

        # set to None if destination_url (nullable) is None
        # and model_fields_set contains the field
        if self.destination_url is None and "destination_url" in self.model_fields_set:
            _dict['DestinationUrl'] = None

        # set to None if tracking_url_template (nullable) is None
        # and model_fields_set contains the field
        if self.tracking_url_template is None and "tracking_url_template" in self.model_fields_set:
            _dict['TrackingUrlTemplate'] = None

        # set to None if final_url_suffix (nullable) is None
        # and model_fields_set contains the field
        if self.final_url_suffix is None and "final_url_suffix" in self.model_fields_set:
            _dict['FinalUrlSuffix'] = None

        # set to None if url_custom_parameters (nullable) is None
        # and model_fields_set contains the field
        if self.url_custom_parameters is None and "url_custom_parameters" in self.model_fields_set:
            _dict['UrlCustomParameters'] = None

        # set to None if final_urls (nullable) is None
        # and model_fields_set contains the field
        if self.final_urls is None and "final_urls" in self.model_fields_set:
            _dict['FinalUrls'] = None

        # set to None if final_mobile_urls (nullable) is None
        # and model_fields_set contains the field
        if self.final_mobile_urls is None and "final_mobile_urls" in self.model_fields_set:
            _dict['FinalMobileUrls'] = None

        # set to None if final_app_urls (nullable) is None
        # and model_fields_set contains the field
        if self.final_app_urls is None and "final_app_urls" in self.model_fields_set:
            _dict['FinalAppUrls'] = None

        # set to None if bidding_scheme (nullable) is None
        # and model_fields_set contains the field
        if self.bidding_scheme is None and "bidding_scheme" in self.model_fields_set:
            _dict['BiddingScheme'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Keyword from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Bid": Bid.from_dict(obj["Bid"]) if obj.get("Bid") is not None else None,
                        "MatchType": obj.get("MatchType") if obj.get("MatchType") is not None else None,
                        "Param1": obj.get("Param1") if obj.get("Param1") is not None else None,
                        "Param2": obj.get("Param2") if obj.get("Param2") is not None else None,
                        "Param3": obj.get("Param3") if obj.get("Param3") is not None else None,
                        "Text": obj.get("Text") if obj.get("Text") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "EditorialStatus": obj.get("EditorialStatus") if obj.get("EditorialStatus") is not None else None,
                        "DestinationUrl": obj.get("DestinationUrl") if obj.get("DestinationUrl") is not None else None,
                        "TrackingUrlTemplate": obj.get("TrackingUrlTemplate") if obj.get("TrackingUrlTemplate") is not None else None,
                        "FinalUrlSuffix": obj.get("FinalUrlSuffix") if obj.get("FinalUrlSuffix") is not None else None,
                        "UrlCustomParameters": CustomParameters.from_dict(obj["UrlCustomParameters"]) if obj.get("UrlCustomParameters") is not None else None,
                        "FinalUrls": obj.get("FinalUrls"),
                        "FinalMobileUrls": obj.get("FinalMobileUrls"),
                        "FinalAppUrls": [AppUrl.from_dict(_item) for _item in obj["FinalAppUrls"]] if obj.get("FinalAppUrls") is not None else None,
                        "BiddingScheme": BiddingScheme.from_dict(obj["BiddingScheme"]) if obj.get("BiddingScheme") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None
        })
        return _obj
